<h1>CSS Grid Layout Module Level 1</h1>

<pre class='metadata'>
Status: WD
Date: 2015-09-17
Work Status: Refining
Shortname: css-grid
Level: 1
Group: csswg
ED: http://dev.w3.org/csswg/css-grid/
TR: http://www.w3.org/TR/css-grid-1/
Previous version: http://www.w3.org/TR/2015/WD-css-grid-1-20150806/
Previous version: http://www.w3.org/TR/2015/WD-css-grid-1-20150317/
Previous version: http://www.w3.org/TR/2014/WD-css-grid-1-20140513/
Previous version: http://www.w3.org/TR/2014/WD-css-grid-1-20140123/
Previous version: http://www.w3.org/TR/2013/WD-css3-grid-layout-20130402/
Previous version: http://www.w3.org/TR/2012/WD-css3-grid-layout-20121106/
Editor: Tab Atkins Jr., Google, http://www.xanthir.com/contact/
Editor: Elika J. Etemad / fantasai, Invited Expert, http://fantasai.inkedblade.net/contact
Editor: Rossen Atanassov, Microsoft, ratan@microsoft.com
Former editor: Alex Mogilevsky, Microsoft Corporation, alexmog@microsoft.com
Former editor: Phil Cupp, Microsoft Corporation, pcupp@microsoft.com
Issue Tracking: Disposition of Comments http://dev.w3.org/csswg/css-grid-1/issues
Abstract: This CSS module defines a two-dimensional grid-based layout system, optimized for user interface design. In the grid layout model, the children of a grid container can be positioned into arbitrary slots in a flexible or fixed predefined layout grid.
Ignored Terms: containing block, <positive-integer>, <ident>, auto, grid-*-start, grid-*-end, flex factor, flex factors, block formatting context, grid-auto-position, min size property, max size property
Link Defaults: css21 (property) margin/min-height/max-height/min-width/max-width, css-align-3 (dfn) alignment container/alignment subject/box alignment properties, css-align-3 (value) stretch/baseline, css-position-3 (property) left, css-position-3 (property) position, css-writing-modes-3 (dfn) start/end
At Risk: the ''grid-template-rows/subgrid'' value of 'grid-template-columns' and 'grid-template-rows', and its component parts individually
</pre>

<style type="text/css">
.example {
	clear:both
}

.pseudo-code {
	font-family:monospace
}
.pseudo-code > ol {
	list-style-type:decimal
}
.pseudo-code > ol > li > ol {
	list-style-type:lower-latin
}
.pseudo-code > ol > li > ol > li > ol {
	list-style-type:lower-roman
}
.pseudo-code ul {
	list-style-type:disc
}

dd > p:nth-child(1) {
	margin-top:0
}

dd:last-child {
	margin-bottom: 0;
}

dl.phase {
	padding-left: .5em;
	border-left: .5em #e0e0e0 solid;
}
#grid-property-breakdown tr:nth-child(3) td {
	width: 25%;
	border-style: solid;
}
</style>

<h2 id='intro'>
Introduction and Overview</h2>

	Grid layout contains features targeted at web application authors.
	The grid can be used to achieve many different layouts.
	It excels at dividing up space for major regions of an application,
	or defining the relationship in terms of size, position, and layer
	between parts of a control built from HTML primitives.

	Like tables,
	grid layout enables an author to align elements into columns and rows,
	but unlike tables,
	grid layout doesn't have content structure,
	and thus enables a wide variety of layouts not possible with tables.
	For example, the children of a grid container can position themselves
	such that they overlap and layer similar to positioned elements.

	In addition, the absence of content structure in grid layout helps to manage changes to layout
	by using fluid and source order independent layout techniques.
	By combining media queries with the CSS properties that control layout of the grid container and its children,
	authors can adapt their layout to changes in device form factors, orientation, and available space,
	without needing to alter the semantic nature of their content.


<h3 id='background'>
Background and Motivation</h3>

	<figure class="sidefigure">
		<img class="figure" alt="Image: Application layout example requiring horizontal and vertical alignment." src="images/basic-form.png" />
		<figcaption>Application layout example requiring horizontal and vertical alignment.</figcaption>
	</figure>

	As websites evolved from simple documents into complex, interactive applications,
	tools for document layout, e.g. floats,
	were not necessarily well suited for application layout.
	By using a combination of tables, JavaScript, or careful measurements on floated elements,
	authors discovered workarounds to achieve desired layouts.
	Layouts that adapted to the available space were often brittle
	and resulted in counter-intuitive behavior as space became constrained.
	As an alternative, authors of many web applications opted for a fixed layout
	that cannot take advantage of changes in the available rendering space on a screen.

	The capabilities of grid layout address these problems.
	It provides a mechanism for authors to divide available space for layout into columns and rows
	using a set of predictable sizing behaviors.
	Authors can then precisely position and size the building block elements of their application
	by into <a>grid areas</a> defined by these columns and rows.
	Figure 1 illustrates a basic layout which can be achieved with grid layout.

<h3 id='adapting-to-available-space'>
Adapting Layouts to Available Space</h3>

	<figure class="sidefigure">
			<img alt="Image: Five grid items arranged according to content size and available space." src="images/game-smaller.png" />

		<figcaption>Five grid items arranged according to content size and available space.</figcaption>
	</figure>

	<figure class="sidefigure">
			<img alt="Image: Growth in the grid due to an increase in available space." src="images/game-larger.png" />

		<figcaption>Growth in the grid due to an increase in available space.</figcaption>
	</figure>

	Grid layout can be used to intelligently reflow elements within a webpage.
	Figure 2 represents a game with five major areas in the layout:
	the game title, stats area, game board, score area, and control area.
	The author's intent is to divide the space for the game such that:

	<ul>
		<li>
			The stats area always appears immediately under the game title.
		<li>
			The game board appears to the right of the stats and title.
		<li>
			The top of the game title and the game board should always align.
		<li>
			The bottom of the game board and the stats area align when the game has reached its minimum height,
			but otherwise the game board will stretch to take advantage of all the screen real-estate available to it.
		<li>
			The score area should align into the column created by the game and stats area,
			while the controls are centered under the board.
	</ul>

	As an alternative to using script to control the absolute position, width, and height of all elements,
	the author can use grid layout,
	as shown in Figure 3.
	The following example shows how an author might achieve all the sizing, placement, and alignment rules declaratively.

	Note that there are multiple ways to specify the structure of the grid
	and to position and size <a>grid items</a>,
	each optimized for different scenarios.
	This example illustrates one that an author may use to define the position and space for each <a>grid item</a>
	using the 'grid-template-rows' and 'grid-template-columns' properties on the <a>grid container</a>,
	and the 'grid-row' and 'grid-column' properties on each <a>grid item</a>.

	<div class="example">
		<pre class="lang-css">
		#grid {
			display: grid;

			/* Two columns: the first sized to content, the second receives
			 * the remaining space, but is never smaller than the minimum
			 * size of the board or the game controls, which occupy this
			 * column. */
			grid-template-columns: auto minmax(min-content, 1fr);

			/* Three rows: the first and last sized to content, the middle
			 * row receives the remaining space, but is never smaller than
			 * the minimum height of the board or stats areas. */
			grid-template-rows: auto minmax(min-content, 1fr) auto
		}

		/* Each part of the game is positioned between grid lines by
		 * referencing the starting grid line and then specifying, if more
		 * than one, the number of rows or columns spanned to determine
		 * the ending grid line, which establishes bounds for the part. */
		#title    { grid-column: 1; grid-row: 1 }
		#score    { grid-column: 1; grid-row: 3 }
		#stats    { grid-column: 1; grid-row: 2; align-self: start }
		#board    { grid-column: 2; grid-row: 1 / span 2; }
		#controls { grid-column: 2; grid-row: 3; justify-self: center }
		</pre>

		<pre class="lang-markup">
			&lt;div id="grid">
			  &lt;div id="title">Game Title&lt;/div>
			  &lt;div id="score">Score&lt;/div>
			  &lt;div id="stats">Stats&lt;/div>
			  &lt;div id="board">Board&lt;/div>
			  &lt;div id="controls">Controls&lt;/div>
			&lt;/div>
		</pre>
	</div>

<h3 id='source-independence'>
Source-Order Independence</h3>

	<figure class="sidefigure">
		<img alt="Image: An arrangement suitable for portrait orientation." src="images/game-portrait.png" />

		<figcaption>An arrangement suitable for “portrait” orientation.</figcaption>
	</figure>

	<figure class="sidefigure">
		<img alt="Image: An arrangement suitable for landscape orientation." src="images/game-landscape.png" />

		<figcaption>An arrangement suitable for “landscape“ orientation.</figcaption>
	</figure>

	Continuing the prior example,
	the author also wants the game to adapt to the space available on traditional computer monitors, handheld devices, or tablet computers.
	Also, the game should optimize the placement of the components when viewed either in portrait or landscape orientation (Figures 4 and 5).
	By combining grid layout with media queries,
	the author is able to use the same semantic markup,
	but rearrange the layout of elements independent of their source order,
	to achieve the desired layout in both orientations.

	The following example leverages grid layout’s ability to name the space which will be occupied by a <a>grid item</a>.
	This allows the author to avoid rewriting rules for <a>grid items</a>
	as the grid’s definition changes.

	<div class="example">
		<pre class="lang-css">
		@media (orientation: portrait) {
			#grid {
				display: grid;

				/* The rows, columns and areas of the grid are defined visually
				* using the grid-template-areas property.  Each string is a row,
				* and each word an area.  The number of words in a string
				* determines the number of columns. Note the number of words
				* in each string must be identical. */
				grid-template-areas: "title stats"
				                     "score stats"
				                     "board board"
				                     "ctrls ctrls";

				/* Columns and rows created with the template property can be
				* assigned a sizing function with the grid-template-columns
				* and grid-template-rows properties. */
				grid-template-columns: auto minmax(min-content, 1fr);
				grid-template-rows: auto auto minmax(min-content, 1fr) auto
			}
		}

		@media (orientation: landscape) {
			#grid {
				display: grid;

				/* Again the template property defines areas of the same name,
				* but this time positioned differently to better suit a
				* landscape orientation. */
				grid-template-areas: "title board"
				                     "stats board"
				                     "score ctrls";

				grid-template-columns: auto minmax(min-content, 1fr);
				grid-template-rows: auto minmax(min-content, 1fr) auto
			}
		}

		/* The grid-area property places a grid item into a named
		* region (area) of the grid. */
		#title    { grid-area: title }
		#score    { grid-area: score }
		#stats    { grid-area: stats }
		#board    { grid-area: board }
		#controls { grid-area: ctrls }
		</pre>
		<pre class="lang-markup">
		&lt;div id="grid">
			&lt;div id="title">Game Title&lt;/div>
			&lt;div id="score">Score&lt;/div>
			&lt;div id="stats">Stats&lt;/div>
			&lt;div id="board">Board&lt;/div>
			&lt;div id="controls">Controls&lt;/div>
		&lt;/div>
		</pre>
	</div>

	Note: The reordering capabilities of grid layout intentionally affect
	<em>only the visual rendering</em>,
	leaving speech order and navigation based on the source order.
	This allows authors to manipulate the visual presentation
	while leaving the source order intact and optimized for non-CSS UAs
	and for linear models such as speech and sequential navigation.

	Advisement: Grid item placement and reordering must not be used
	as a substitute for correct source ordering,
	as that can ruin the accessibility of the document.

<h3 id='grid-layering'>
Grid Layering of Elements</h3>

	<figure class="sidefigure">
		<img alt="Image: A control composed of layered HTML elements." src="images/control-layering-and-alignment.png" />

		<figcaption>A control composed of layered HTML elements.</figcaption>
	</figure>

	In the example shown in Figure 6,
	the author is creating a custom slider control.
	The control has six parts.
	The lower and upper labels align to the left and right edges of the control.
	The track of the slider spans the area between the labels.
	The lower and upper fill parts touch beneath the thumb,
	and the thumb is a fixed width and height that can be moved along the track
	by updating the two flex-sized columns.

	Prior to the introduction of grid layout,
	the author would have likely used absolute positioning to control the top and left coordinates,
	along with the width and height of each HTML element that comprises the control.
	By leveraging grid layout,
	the author can instead limit script usage to handling mouse events on the thumb,
	which snaps to various positions along the track
	as the 'grid-template-columns' property of the <a>grid container</a> is updated.

	<div class="example">
		<pre class="lang-css">
		#grid {
			display: grid;

			/* The grid-template-columns and rows properties also support
			* naming grid lines which can then be used to position grid
			* items.  The line names are assigned on either side of a column
			* or row sizing function where the line would logically exist. */
			grid-template-columns:
				[start]        auto
				[track-start]  0.5fr
				[thumb-start]  auto
				[fill-split]   auto
				[thumb-end]    0.5fr
				[track-end]    auto
				[end];
		}

		/* The grid-placement properties accept named lines. Below the
		* lines are referred to by name. Beyond any
		* semantic advantage, the names also allow the author to avoid
		* renumbering the grid-column-start and grid-row-start properties of the
		* grid items.  This is similar to the concept demonstrated in the
		* prior example with the grid-template-areas property during orientation
		* changes, but grid lines can also work with layered grid items
		* that have overlapping areas of different shapes like the thumb
		* and track parts in this example. */
		#lower-label { grid-column-start: start }
		#track       { grid-column: track-start / track-end; justify-self: center }
		#upper-label { grid-column-end: end; }

		/* Fill parts are drawn above the track so set z-index to 5. */
		#lower-fill  { grid-column: track-start / fill-split;
		               justify-self: end;
		               z-index: 5 }
		#upper-fill  { grid-column: fill-split / track-end;
		               justify-self: start;
		               z-index: 5 }

		/* Thumb is the topmost part; assign it the highest z-index value. */
		#thumb       { grid-column: thumb-start / thumb-end; z-index: 10 }
		</pre>
		<pre class="lang-markup">
		&lt;div id="grid">
			&lt;div id="lower-label">Lower Label&lt;/div>
			&lt;div id="upper-label">Upper Label&lt;/div>
			&lt;div id="track">Track&lt;/div>
			&lt;div id="lower-fill">Lower Fill&lt;/div>
			&lt;div id="upper-fill">Upper Fill&lt;/div>
			&lt;div id="thumb">Thumb&lt;/div>
		&lt;/div>
		</pre>
	</div>

<!--
 ██████   ███████  ██    ██  ██████  ████████ ████████  ████████  ██████
██    ██ ██     ██ ███   ██ ██    ██ ██       ██     ██    ██    ██    ██
██       ██     ██ ████  ██ ██       ██       ██     ██    ██    ██
██       ██     ██ ██ ██ ██ ██       ██████   ████████     ██     ██████
██       ██     ██ ██  ████ ██       ██       ██           ██          ██
██    ██ ██     ██ ██   ███ ██    ██ ██       ██           ██    ██    ██
 ██████   ███████  ██    ██  ██████  ████████ ██           ██     ██████
-->

<h2 id='grid-concepts'>
Grid Layout Concepts and Terminology</h2>

	In <dfn export>grid layout</dfn>,
	the content of a <a>grid container</a> is laid out
	by positioning and aligning it into a <a>grid</a>.
	The <dfn export>grid</dfn> is an intersecting set of horizontal and vertical <a>grid lines</a>
	that divides the <a>grid container</a>’s space into <a>grid areas</a>,
	into which <a>grid items</a> (representing the <a>grid container</a>’s content) can be placed.
	There are two sets of <a>grid lines</a>:
	one set defining <dfn export lt="grid column | column">columns</dfn>
	that run along the <a href="http://www.w3.org/TR/css3-writing-modes/#block-axis-">block axis</a> (the <dfn export>column axis</dfn>),
	and an orthogonal set defining <dfn export lt="grid row | row">rows</dfn>
	along the <a href="http://www.w3.org/TR/css3-writing-modes/#inline-axis-">inline axis</a> (the <dfn export>row axis</dfn>).
	[[!CSS3-WRITING-MODES]]

<!--
	<div class="figure">
		<img class="figure" alt="Image: A diagram illustrating the relationship between the Grid Element and its Tracks, Lines, Areas and Items." src="images/grid-concepts.png" />
		<p class="caption">A diagram illustrating the relationship between the <a>grid container</a> and its tracks, lines, areas and items.
	</div>
-->

	<figure>
		<img class="figure" alt="Image: Grid Lines." src="images/grid-lines.png" />
		<figcaption>Grid lines: Three in the block axis and four in the inline axis.</figcaption>
	</figure>

<h3 id="grid-track-concept">
Grid Tracks and Cells</h3>

	<dfn export>Grid track</dfn> is a generic term for a <a>grid column</a> or <a>grid row</a>&mdash;in
	other words, it is the space between two adjacent <a>grid lines</a>.
	Each <a>grid track</a> is assigned a sizing function,
	which controls how wide or tall the column or row may grow,
	and thus how far apart its bounding <a>grid lines</a> are.

	A <dfn export>grid cell</dfn> is the similar term for the full grid&mdash;it
	is the space between two adjacent row and two adjacent column <a>grid lines</a>.
	It is the smallest unit of the grid that can be referenced when positioning <a>grid items</a>.

	<div class="example">
		In the following example there are two columns and three rows.
		The first column is fixed at 150px.
		The second column uses flexible sizing, which is a function of the unassigned space in the Grid,
		and thus will vary as the width of the <a>grid container</a> changes.
		If the used width of the <a>grid container</a> is 200px, then the second column 50px wide.
		If the used width of the <a>grid container</a> is 100px, then the second column is 0px
		and any content positioned in the column will overflow the <a>grid container</a>.

		<pre>
			&lt;style type="text/css">
				#grid {
					display: grid;
					grid-template-columns: 150px 1fr; /* two columns */
					grid-template-rows: 50px 1fr 50px /* three rows  */
				}
			&lt;/style>
		</pre>
	</div>

<h3 id="grid-line-concept">
Grid Lines</h3>

	<dfn id='grid-line' export lt='grid line|grid row line|grid column line'>Grid lines</dfn> are the horizontal and vertical dividing lines of the <a>grid</a>.
	A <a>grid line</a> exists on either side of a column or row.
	They can be referred to by numerical index,
	or by an author-specified name.
	A <a>grid item</a> references the <a>grid lines</a> to determine its position within the <a>grid</a>
	using the <a href="#placement">grid-placement properties</a>.

	<div class="example">
		The following two examples create three column <a>grid lines</a> and four row <a>grid lines</a>.
		The first example demonstrates how an author would position a <a>grid item</a> using <a>grid line</a> numbers.
		The second example uses explicitly named <a>grid lines</a>.

		<pre>
			&lt;style type="text/css">
			#grid {
				display: grid;
				grid-template-columns: 150px 1fr;
				grid-template-rows: 50px 1fr 50px
			}

			#item1 { grid-column: 2;
			         grid-row-start: 1; grid-row-end: 4; }
			&lt;/style>
		</pre>

		<pre>
			&lt;style type="text/css">
			/* equivalent layout to the prior example, but using named lines */
			#grid {
				display: grid;
				grid-template-columns: 150px [item1-start] 1fr [item1-end];
				grid-template-rows: [item1-start] 50px 1fr 50px [item1-end];
			}

			#item1 {
				grid-column: item1-start / item1-end;
				grid-row: item1-start / item1-end
			}
			&lt;/style>
		</pre>
	</div>


<h3 id="grid-area-concept">
Grid Areas</h3>

	A <dfn export>grid area</dfn> is the logical space used to lay out one or more <a>grid items</a>.
	It is bound by four <a>grid lines</a>, one on each side of the <a>grid area</a>,
	and participates in the sizing of the <a>grid tracks</a> it intersects.
	A <a>grid area</a> can be named explicitly using the 'grid-template-areas' property of the <a>grid container</a>,
	or referenced implicitly by its bounding <a>grid lines</a>.
	A <a>grid item</a> is assigned to a <a>grid area</a>
	using the <a href="#placement">grid-placement properties</a>.

	<div class="example">
		<pre>
			&lt;style type="text/css">
			/* using the template syntax */
			#grid  {
			display: grid;
			grid-template-areas: ". a"
			                     "b a"
			                     ". a";
			grid-template-columns: 150px 1fr;
			grid-template-rows: 50px 1fr 50px
			}

			#item1 { grid-area: a }
			#item2 { grid-area: b }
			#item3 { grid-area: b }

			/* Align items 2 and 3 at different points in the Grid Area "b".  */
			/* By default, Grid Items are stretched to fit their Grid Area    */
			/* and these items would layer one over the other. */
			#item2 { align-self: start }
			#item3 { justify-self: end; align-self: end }
			&lt;/style>
		</pre>
	</div>

	A <a>grid item</a>’s <a>grid area</a> forms the containing block into which it is laid out.
	Percentage lengths specified on a <a>grid item</a> resolve against this containing block.
	Percentages specified for 'margin-top', 'padding-top', 'margin-bottom', and 'padding-bottom' on a <a>grid item</a>
	resolve against the height of its containing block,
	rather than the width (as for blocks).

	Issue: This margin/padding behavior is disputed.
	<a href="https://lists.w3.org/Archives/Public/www-style/2015Mar/0273.html">(see discussion)</a>

	<a>Grid items</a> placed into the same <a>grid area</a> do not directly affect each other's layout.
	Indirectly, a <a>grid item</a> can affect the position of a <a>grid line</a> in a column or row that uses a content-based relative size,
	which in turn can affect the position or size of another <a>grid item</a>.

<!--
████████  ████  ██████  ████████  ██          ███    ██    ██  ██   ██████   ████████  ████ ████████
██     ██  ██  ██    ██ ██     ██ ██         ██ ██    ██  ██  ████ ██    ██  ██     ██  ██  ██     ██
██     ██  ██  ██       ██     ██ ██        ██   ██    ████    ██  ██        ██     ██  ██  ██     ██
██     ██  ██   ██████  ████████  ██       ██     ██    ██         ██   ████ ████████   ██  ██     ██
██     ██  ██        ██ ██        ██       █████████    ██     ██  ██    ██  ██   ██    ██  ██     ██
██     ██  ██  ██    ██ ██        ██       ██     ██    ██    ████ ██    ██  ██    ██   ██  ██     ██
████████  ████  ██████  ██        ████████ ██     ██    ██     ██   ██████   ██     ██ ████ ████████
-->

<h2 id="grid-model">
Grid Containers</h2>

<h3 id='grid-containers'>
Establishing Grid Containers: the ''display/grid'' and ''inline-grid'' 'display' values</h3>

	<pre class="propdef">
	Name: display
	New values: grid | inline-grid
	</pre>

	<dl dfn-for="display" dfn-type=value>
		<dt><dfn>grid</dfn>
		<dd>
			This value causes an element to generate a block-level <a>grid container</a> box.

		<dt><dfn>inline-grid</dfn>
		<dd>
			This value causes an element to generate an inline-level <a>grid container</a> box.
	</dl>

	A <dfn export>grid container</dfn> establishes a new <dfn export>grid formatting context</dfn> for its contents.
	This is the same as establishing a block formatting context,
	except that grid layout is used instead of block layout:
	floats do not intrude into the grid container,
	and the grid container's margins do not collapse with the margins of its contents.
	The contents of a <a>grid container</a> are laid out into a <a>grid</a>,
	with <a>grid lines</a> forming the boundaries of each <a>grid items</a>’ containing block.
	The 'overflow' property applies to <a>grid containers</a>.

	Grid containers are not block containers,
	and so some properties that were designed with the assumption of block layout
	don't apply in the context of grid layout.
	In particular:

	<ul>
		<li>
			the 'column-*' properties in the Multi-column Layout module [[!CSS3COL]] have no effect on a grid container.

		<li>
			'float' and 'clear' have no effect on a <a>grid item</a>.
			(However, the 'float' property still affects the computed value of 'display' on children of a grid container,
			as this occurs <em>before</em> <a>grid items</a> are determined.)

		<li>
			'vertical-align' has no effect on a grid item.

		<li>
			the ''::first-line'' and ''::first-letter'' pseudo-elements do not apply to <a>grid containers</a>,
			and <a>grid containers</a> do not contribute a first formatted line or first letter to their ancestors.
	</ul>

	If an element's specified 'display' is ''inline-grid''
	and the element is floated or absolutely positioned,
	the computed value of 'display' is <a value>grid</a>.
	The table in <a href="http://www.w3.org/TR/CSS2/visuren.html#dis-pos-flo">CSS 2.1 Chapter 9.7</a> is thus amended
	to contain an additional row,
	with ''inline-grid'' in the "Specified Value" column
	and ''grid'' in the "Computed Value" column.

<h3 id='intrinsic-sizes'>
Sizing Grid Containers</h3>

	A <a>grid container</a> is sized
	using the rules of the formatting context in which it participates.
	As a block-level box in a <a>block formatting context</a>,
	it is sized like any other block-level box that establishes a formatting context,
	with an ''auto'' inline size calculated as for in-flow block boxes.
	As an inline-level box in an inline formatting context,
	it is sized as an atomic inline-level box (such as an inline-block).
	In both inline and block formatting contexts,
	the <a>grid container</a>’s ''auto'' block size is its max-content size.
	<span class="issue">The block layout spec should define this?</span>

	The <a>max-content size</a> of a <a>grid container</a> is
	the sum of the <a>grid container’s</a> track sizes in the appropriate axis,
	when the grid is sized under a <a>max-content constraint</a>.

	The <a>min-content size</a> of a <a>grid container</a> is
	the sum of the <a>grid container’s</a> track sizes in the appropriate axis,
	when the grid is sized under a <a>min-content constraint</a>.

	See [[!CSS3-SIZING]] for a definition of the terms in this section.

<!--
 ██████   ████████  ████ ████████        ████ ████████ ████████ ██     ██  ██████
██    ██  ██     ██  ██  ██     ██        ██     ██    ██       ███   ███ ██    ██
██        ██     ██  ██  ██     ██        ██     ██    ██       ████ ████ ██
██   ████ ████████   ██  ██     ██        ██     ██    ██████   ██ ███ ██  ██████
██    ██  ██   ██    ██  ██     ██        ██     ██    ██       ██     ██       ██
██    ██  ██    ██   ██  ██     ██        ██     ██    ██       ██     ██ ██    ██
 ██████   ██     ██ ████ ████████        ████    ██    ████████ ██     ██  ██████
-->

<h2 id="grid-items">
Grid Items</h2>

	The contents of a <a>grid container</a> consists of zero or more <dfn export id="grid-item" lt="grid item">grid items</dfn>:
	each child of a <a>grid container</a>
	becomes a <a>grid item</a>,
	and each contiguous run of text that is directly contained inside a <a>grid container</a>
	is wrapped in an anonymous <a>grid item</a>.
	However, an anonymous grid item that contains only
	<a href="http://www.w3.org/TR/css-text/#white-space-processing">white space</a>
	is not rendered, as if it were ''display:none''.

	<div class="example">
			Examples of grid items:
		<pre>
&lt;div style="display:grid">

	&lt;!-- grid item: block child -->
	&lt;div id="item1">block&lt;/div>

	&lt;!-- grid item: floated element; floating is ignored -->
	&lt;div id="item2" style="float: left;">float&lt;/div>

	&lt;!-- grid item: anonymous block box around inline content -->
	anonymous item 3

	&lt;!-- grid item: inline child -->
	&lt;span>
		item 4
		&lt;!-- grid items do not split around blocks -->
		&lt;div id=not-an-item>item 4&lt;/div>
		item 4
	&lt;/span>
&lt;/div></pre>
	</div>

	The 'display' value of a <a>grid item</a> is <a>blockified</a>:
	if the specified 'display' of an in-flow child of an element generating a <a>grid container</a>
	is an inline-level value, it computes to its block-level equivalent.
	(See <a href="http://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo">CSS2.1&sect;9.7</a> [[!CSS21]]
	and <a href="http://www.w3.org/TR/css-display/#transformations">CSS Display</a> [[!CSS3-DISPLAY]]
	for details on this type of 'display' value conversion.)

	Some values of 'display' trigger the generation of anonymous boxes.
	For example, a misparented ''display/table-cell'' child is fixed up
	by <a href="http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes">generating anonymous <css>table</css> and <css>table-row</css> elements</a> around it. [[!CSS21]]
	This fixup must occur <em>before</em> a <a>grid container</a>’s children are promoted to <a>grid items</a>.
	For example, given two contiguous child elements with ''display:table-cell'',
	an anonymous table wrapper box around them becomes the <a>grid item</a>.

	<p class='note'>
		Future display types may generate anonymous containers (e.g. ruby) or otherwise mangle the box tree (e.g. run-ins).
		It is intended that grid item determination run after these operations.

	A <a>grid item</a> establishes a new formatting context for its contents.
	The type of this formatting context is determined by its 'display' value, as usual.
	The computed 'display' of a <a>grid item</a>
	is determined by applying the table in
	<a href="http://www.w3.org/TR/CSS2/visuren.html#dis-pos-flo">CSS 2.1 Chapter 9.7</a>.
	However, grid items are <dfn>grid-level</dfn> boxes, not block-level boxes:
	they participate in their container's <a>grid formatting context</a>,
	not in a block formatting context.

	A <a>grid item</a> is sized within the containing block defined by its <a>grid area</a>
	similarly to an equivalent block-level box in an equivalently-sized containing block,
	except that ''margin/auto'' margins and the <a>box alignment properties</a>
	have special effects. (See [[#alignment]].)

	The ''min-width/auto'' value of 'min-width' and 'min-height'
	behaves on <a>grid items</a> in the relevant axis
	analogously to its behavior on <a>flex items</a> in the <a>main axis</a>.
	See [[#min-size-auto]].

	ISSUE: Review implications of intrinsic ratio and Grid's 2D nature.

<!--
<h3 id="position-grid">
Non-children Grid Items</h3>

	<p class="issue">
		This is a proposal to create the ability to have descendants of a grid item participate in a grid layout,
		similar to the behavior defined by the Template Layout module.

		A descendant of the grid can be pulled out of flow and participate directly in the grid
		by assigning it ''position: grid''.
		An element with ''position: grid'' is pulled out of flow and participates as a grid item
		belonging to the first ancestor with ''display: grid''.
		If the element is positioned using named lines or slots,
		it belongs to the first ancestor with ''display: grid'' that has all of the corresponding named lines/slots.
		If no such ancestor exists, the item remains in flow.

	<p class="issue">
		Alternatively, the item can just go into the first grid,
		and missing names are treated as <a value for="<grid-line>">auto</a>.
-->

<h3 id="visibility-collapse">
	Collapsed Grid Items: the 'visibility' property</h3>

	<p class='issue'>
		We want the ability to collapse grid tracks
		(similar to <a href="http://www.w3.org/TR/css3-flexbox/#visibility-collapse">collapsing flex items</a>
		 or <a href="http://www.w3.org/TR/CSS21/tables.html#dynamic-effects">table rows/columns</a>),
		but we're not sure exactly how to do it.
		Ideas welcome, please <a href="mailto:www-style@w3.org?Subject=%5Bcss-grid%5D%20Collapsing%20Grid%20Tracks">post them to www-style@w3.org</a>.

<h3 id='order-property'>
Reordered Grid Items: the 'order' property</h3>

	The 'order' property also applies to <a>grid items</a>.
	It affects their <a href="#grid-auto-flow-property">auto-placement</a> and <a href="#z-order">painting order</a>.

	Advisement: As with reordering flex items,
	the 'order' property must only be used
	when the visual order needs to be <em>out-of-sync</em>
	with the speech and navigation order;
	otherwise the underlying document source should be reordered instead.
	See <a href="https://drafts.csswg.org/css-flexbox-1/#order-accessibility">Reordering and Accessibility</a>
	in [[CSS-FLEXBOX-1]].

<h3 id='z-order'>
Z-axis Ordering: the 'z-index' property</h3>

	<a>Grid items</a> can overlap when they are positioned into intersecting <a>grid areas</a>,
	or even when positioned in non-intersecting areas because of negative margins or positioning.
	The painting order of <a>grid items</a> is exactly the same as inline blocks [[CSS21]],
	except that <a>order-modified document order</a> is used in place of raw document order,
	and 'z-index' values other than <a value for=z-index>auto</a> create a stacking context even if 'position' is ''static''.
	Thus the 'z-index' property can easily be used to control the z-axis order of grid items.

	<p class='note'>
		Note: Descendants that are positioned outside a grid item still participate in any stacking context established by the grid item.

	<div class="example">
		The following diagram shows several overlapping grid items,
		with a combination of implicit source order
		and explicit 'z-index'
		used to control their stacking order.

		<figure>
			<img src="images/drawing-order.png" />

			<figcaption>Drawing order controlled by z-index and source order.</figcaption>
		</figure>

		<pre>
			&lt;style type="text/css">
			#grid {
				display: grid;
				grid-template-columns: 1fr 1fr;
				grid-template-rows: 1fr 1fr
			}
			#A { grid-column: 1 / span 2; grid-row: 2; align-self: end; }
			#B { grid-column: 1; grid-row: 1; z-index: 10; }
			#C { grid-column: 2; grid-row: 1; align-self: start; margin-left: -20px; }
			#D { grid-column: 2; grid-row: 2; justify-self: end; align-self: start; }
			#E { grid-column: 1 / span 2; grid-row: 1 / span 2;
			     z-index: 5; justify-self: center; align-self: center; }
			&lt;/style>

			&lt;div id="grid">
				&lt;div id="A">A&lt;/div>
				&lt;div id="B">B&lt;/div>
				&lt;div id="C">C&lt;/div>
				&lt;div id="D">D&lt;/div>
				&lt;div id="E">E&lt;/div>
			&lt;/div>
		</pre>
	</div>

<!--
██     ██ ████ ██    ██
███   ███  ██  ███   ██          ██   ██
████ ████  ██  ████  ██           ██ ██
██ ███ ██  ██  ██ ██ ██ ███████ █████████
██     ██  ██  ██  ████           ██ ██
██     ██  ██  ██   ███          ██   ██
██     ██ ████ ██    ██
-->

<h3 id="min-size-auto">
Implied Minimum Size of Grid Items</h3>

	To provide a more reasonable default minimum size for <a>grid items</a>,
	this specification defines the effects of the 'min-width'/'min-height' ''min-width/auto'' value
	for <a>grid items</a>.

	On a <a>grid item</a> whose 'overflow' is ''overflow/visible'',
	when ''min-width/auto'' is specified on the <a>grid item</a>,
	the following table gives the minimum size in that dimension:

	<table class=data>
		<colgroup span=2></colgroup>
		<colgroup span=1></colgroup>
		<thead>
			<tr>
				<th>Specified Size
				<th>Transferred Size
				<th>Minimum Size
		<tbody>
			<tr>
				<td>
				<td>
				<td><var>content size</var>
			<tr>
				<td>✓
				<td>
				<td>min(<var>specified size</var>, <var>content size</var>)
			<tr>
				<td>
				<td>✓
				<td>min(<var>transferred size</var>, <var>content size</var>)
			<tr>
				<td>✓
				<td>✓
				<td>min(<var>specified size</var>, <var>content size</var>)
	</table>

	Where:

	<dl>
		<dt><var>specified size</var>
		<dd>
			If the item’s computed width/height is <a>definite</a>,
			then the <var>specified size</var> is that size
			(clamped by its <a>max size property</a> in that dimension if it's <a>definite</a>).
			It is otherwise undefined.

		<dt><var>transferred size</var>
		<dd>
			If the item has an intrinsic aspect ratio
			and its computed height/width is <a>definite</a>,
			then the <var>transferred size</var> is that size
			(clamped by its <a lt="min size property">min and max size properties</a> in that dimension if they are <a>definite</a>),
			converted through the aspect ratio.
			It is otherwise undefined.

		<dt><var>content size</var>
		<dd>
			The <var>content size</var> is the <a>min-content size</a> in that dimension,
			clamped, if it has an aspect ratio, by any <a>definite</a> <a lt="min size property">min and max size properties</a>
			in the perpendicular dimension
			converted through the aspect ratio,
			and then further clamped by the <a>max size property</a> in the relevant dimension if that is <a>definite</a>.
	</dl>


	<div class="note" id="min-size-opt">
		Note that while a content-based minimum size is often appropriate,
		and helps prevent content from overlapping or spilling outside its container,
		in some cases it is not:

		In particular, if flex sizing is being used for a major content area of a document,
		it is better to set an explicit font-relative minimum width such as ''min-width: 12em''.
		A content-based minimum width could result in a large table or large image
		stretching the size of the entire content area into an overflow zone,
		and thereby making lines of text gratuitously long and hard to read.

		Note also, when content-based sizing is used on an item with large amounts of content,
		the layout engine must traverse all of this content before finding its minimum size,
		whereas if the author sets an explicit minimum, this is not necessary.
		(For items with small amounts of content, however,
		this traversal is trivial and therefore not a performance concern.)
	</div>

<!--
████████ ██     ██ ████████  ██       ████  ██████  ████ ████████
██        ██   ██  ██     ██ ██        ██  ██    ██  ██     ██
██         ██ ██   ██     ██ ██        ██  ██        ██     ██
██████      ███    ████████  ██        ██  ██        ██     ██
██         ██ ██   ██        ██        ██  ██        ██     ██
██        ██   ██  ██        ██        ██  ██    ██  ██     ██
████████ ██     ██ ██        ████████ ████  ██████  ████    ██
-->

<h2 id='grid-definition'>
The Explicit Grid</h2>

	The three properties 'grid-template-rows', 'grid-template-columns', and 'grid-template-areas'
	together define the <dfn export local-lt="explicit">explicit grid</dfn> of a <a>grid container</a>.
	The 'grid-template' property is a shorthand that sets all three at the same time.
	The final grid may end up larger due to <a>grid items</a> placed outside the <a>explicit grid</a>;
	in this case, any implicit tracks are sized by the 'grid-auto-rows' and 'grid-auto-columns' properties.

	The size of the <a>explicit grid</a> is determined by the larger of
	the number of rows/columns defined by 'grid-template-areas'
	and the number of rows/columns sized by 'grid-template-rows'/'grid-template-columns'.
	Any rows/columns defined by 'grid-template-areas' but not sized by 'grid-template-rows'/'grid-template-columns'
	take their size from the 'grid-auto-rows'/'grid-auto-columns' properties.
	If these properties don't define <em>any</em> <a>explicit</a> tracks,
	the <a>explicit grid</a> still contains one <a>grid line</a> in each axis.

	Numeric indexes in the <a>grid-placement properties</a>
	count from the edges of the <a>explicit grid</a>.
	Positive indexes count from the <a>start</a> side
	(starting from 1 for the <a>start</a>-most <a>explicit</a> line),
	while negative indexes count from the <a>end</a> side
	(starting from -1 for the <a>end</a>-most <a>explicit</a> line).

<h3 id='track-sizing'>
Track Sizing: the 'grid-template-rows' and 'grid-template-columns' properties</h3>

	<pre class='propdef'>
	Name: grid-template-columns, grid-template-rows
	Value: none | <<track-list>> | <<auto-track-list>> | subgrid <<line-name-list>>?
	Initial: none
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: refer to corresponding dimension of the content area
	Media: visual
	Computed value: As specified, with lengths made absolute
	</pre>

	These properties specify,
	as a space-separated <dfn export>track list</dfn>,
	the line names and <a>track sizing functions</a> of the <a>grid</a>.
	Each <dfn lt="track sizing function|sizing function">track sizing function</dfn> can be specified as a length,
	a percentage of the <a>grid container</a>’s size,
	a measurement of the contents occupying the column or row,
	or a fraction of the free space in the grid.
	It can also be specified as a range using the ''minmax()'' notation,
	which can combine any of the previously mentioned mechanisms
	to specify separate <a lt="min track sizing function">min</a>
	and <a>max track sizing functions</a> for the column or row.

	The 'grid-template-columns' property specifies the <a>track list</a> for the grid's columns,
	while 'grid-template-rows' specifies the <a>track list</a> for the grid's rows.

	The <dfn value for="grid-template-rows, grid-template-columns">none</dfn> value indicates that there is no <a>explicit grid</a>;
	any rows/columns will be implicitly generated,
	and their size will be determined by the 'grid-auto-rows' and 'grid-auto-columns' properties.

	The <dfn value for="grid-template-rows, grid-template-columns">subgrid</dfn> value indicates that the grid will align to its parent grid in that axis.
	Rather than specifying the sizes of rows/columns explicitly,
	they'll be taken from the parent grid's definition.

	The syntax of a <a>track list</a> is:

	<pre>
		<dfn>&lt;track-list></dfn>       = [ <<line-names>>? [ <<track-size>> | <<track-repeat>> ] ]+ <<line-names>>?
		<dfn>&lt;auto-track-list></dfn>  = [ <<line-names>>? [ <<fixed-size>> | <<fixed-repeat>> ] ]+ <<auto-repeat>>
		                     [ <<line-names>>? [ <<fixed-size>> | <<fixed-repeat>> ] ]+ <<line-names>>?
		<dfn>&lt;line-name-list></dfn>   = [ <<line-names>> | <<name-repeat>> ]+

		<dfn>&lt;track-size></dfn>       = <<track-breadth>> | minmax( <<track-breadth>> , <<track-breadth>> )
		<dfn>&lt;fixed-size></dfn>       = <<fixed-breadth>> | minmax( <<fixed-breadth>> , <<track-breadth>> )
		<dfn>&lt;track-breadth></dfn>    = <<length>> | <<percentage>> | <<flex>> | min-content | max-content | auto
		<dfn>&lt;fixed-breadth></dfn>    = <<length>> | <<percentage>>
		<dfn>&lt;line-names></dfn>       = '[' <<custom-ident>>* ']'
	</pre>

	Where:

	<dl dfn-for="grid-template-columns, grid-template-rows" dfn-type=value>
		<dt><dfn><<length>></dfn>
		<dd>
			A non-negative length, as defined by CSS3 Values. [[!CSS3VAL]]

		<dt><dfn><<percentage>></dfn>
		<dd>
			A non-negative percentage, as defined by CSS3 Values. [[!CSS3VAL]]
			<<percentage>> values are relative to the <a>inline size</a> of the <a>grid container</a> in column <a>grid tracks</a>,
			and the <a>block size</a> of the <a>grid container</a> in row <a>grid tracks</a>.
			If the <a lt="inline size">inline</a> or <a>block size</a> of the <a>grid container</a>
			is <a>indefinite</a>,
			<<percentage>> values relative to that size are treated as <a value for=width>auto</a>.

		<dt><dfn><<flex>></dfn>
		<dd>
			A non-negative dimension with the unit ''fr'' specifying the track's <dfn dfn noexport>flex factor</dfn>.
			Each <<flex>>-sized track takes a share of the remaining space in proportion to its <a>flex factor</a>.
			See <a href="#fr-unit">Flexible Lengths</a> for more details.

			When appearing outside a ''minmax()'' notation,
			implies an automatic minimum (i.e. ''minmax(auto, <<flex>>)'').

		<dt><dfn>max-content</dfn>
		<dd>
			Represents the largest <a>max-content contribution</a>
			of the <a>grid items</a> occupying the <a>grid track</a>.

		<dt><dfn>min-content</dfn>
		<dd>
			Represents the largest <a>min-content contribution</a>
			of the  <a>grid items</a> occupying the <a>grid track</a>.

		<dt><dfn lt="minmax()">minmax(min, max)</dfn>
		<dd>
			Defines a size range
			greater than or equal to <var>min</var>
			and less than or equal to <var>max</var>.
			If <var>max</var> &lt; <var>min</var>,
			then <var>max</var> is ignored and ''minmax(min,max)'' is treated as <var>min</var>.
			As a maximum, a <<flex>> value sets the track's <a>flex factor</a>.
			As a minimum, it is treated as zero
			(or ''min-content'', if the <a>grid container</a> is sized under a <a>min-content constraint</a>).

		<dt><dfn>auto</dfn>
		<dd>
			As a maximum, identical to ''max-content''.
			As a minimum, represents the largest minimum size (as specified by 'min-width'/'min-height')
			of the <a>grid items</a> occupying the <a>grid track</a>.

			Note: ''grid-template-rows/auto'' track sizes (and only ''grid-template-rows/auto'' track sizes)
			can be streched by the 'align-content' and 'justify-content' properties.
	</dl>

	<div class='example'>
		Given the following 'grid-template-columns' declaration:

		<pre>grid-template-columns: 100px 1fr max-content minmax(min-content, 1fr);</pre>

		Five grid lines are created:

		<ol>
			<li>
				At the start edge of the <a>grid container</a>.

			<li>
				100px from the start edge of the <a>grid container</a>.

			<li>
				A distance from the previous line equal to half the <a>free space</a>
				(the width of the <a>grid container</a>, minus the width of the non-flexible <a>grid tracks</a>).

			<li>
				A distance from the previous line equal to the maximum size of any <a>grid items</a>
				belonging to the column between these two lines.

			<li>
				A distance from the previous line at least as large as the largest minimum size of any <a>grid items</a>
				belonging to the column between these two lines,
				but no larger than the other half of the <a>free space</a>.
		</ol>

		If the non-flexible sizes
		(''100px'', ''max-content'', and ''min-content'')
		sum to larger than the <a>grid container</a>’s width,
		the final <a>grid line</a> will be a distance equal to their sum away from the start edge of the <a>grid container</a>
		(the ''1fr'' sizes both resolve to ''0'').
		If the sum is less than the <a>grid container</a>’s width,
		the final <a>grid line</a> will be exactly at the end edge of the <a>grid container</a>.
		This is true in general whenever there's at least one <<flex>> value among the <a>grid track</a> sizes.
	</div>

	<div class='example'>
		Additional examples of valid <a>grid track</a> definitions:

		<pre>
			/* examples of valid track definitions */
			grid-template-rows: 1fr minmax(min-content, 1fr);
			grid-template-rows: 10px repeat(2, 1fr auto minmax(30%, 1fr));
			grid-template-rows: calc(4em - 5px)
		</pre>
	</div>

	Note: The size of the grid is not purely the sum of the track sizes,
	as 'grid-row-gap', 'grid-column-gap' and  'justify-content', 'align-content'
	can add additional space between tracks.

<!--
██    ██    ███    ██     ██ ████████ ████████        ██       ████ ██    ██ ████████  ██████
███   ██   ██ ██   ███   ███ ██       ██     ██       ██        ██  ███   ██ ██       ██    ██
████  ██  ██   ██  ████ ████ ██       ██     ██       ██        ██  ████  ██ ██       ██
██ ██ ██ ██     ██ ██ ███ ██ ██████   ██     ██       ██        ██  ██ ██ ██ ██████    ██████
██  ████ █████████ ██     ██ ██       ██     ██       ██        ██  ██  ████ ██             ██
██   ███ ██     ██ ██     ██ ██       ██     ██       ██        ██  ██   ███ ██       ██    ██
██    ██ ██     ██ ██     ██ ████████ ████████        ████████ ████ ██    ██ ████████  ██████
-->

<h4 id='named-lines'>
Named Grid Lines: the <css>[<<custom-ident>>*]</css> syntax</h4>

	While <a>grid lines</a> can always be referred to by their numerical index,
	<dfn export lt="named line">named lines</dfn>
	can make the <a>grid-placement properties</a> easier to understand and maintain.
	Lines can be explicitly named in the 'grid-template-rows' and 'grid-template-columns' properties,
	or <a href="#implicit-named-lines">implicitly named</a> by creating <a>named grid areas</a> with the 'grid-template-areas' property.

	<div class='example'>
		For example,
		the following code gives meaningful names to all of the lines in the grid.
		Note that some of the lines have multiple names.

		<pre>
			&lt;style>
			#grid {
				display: grid;
				grid-template-columns: [first nav] 150px [main] 1fr [last];
				grid-template-rows: [first header] 50px [main] 1fr [footer] 50px [last];
			}
			&lt;/style>
		</pre>

		<figure class="figure">
			<img class="figure" alt="Image: Named Grid Lines." src="images/grid-named-lines.png" />
			<figcaption>Named Grid Lines.</figcaption>
		</figure>
	</div>

<!--
████████  ████████ ████████  ████████    ███    ████████   ███ ███
██     ██ ██       ██     ██ ██         ██ ██      ██     ██     ██
██     ██ ██       ██     ██ ██        ██   ██     ██    ██       ██
████████  ██████   ████████  ██████   ██     ██    ██    ██       ██
██   ██   ██       ██        ██       █████████    ██    ██       ██
██    ██  ██       ██        ██       ██     ██    ██     ██     ██
██     ██ ████████ ██        ████████ ██     ██    ██      ███ ███
-->

<h4 id='repeat-notation'>
Repeating Rows and Columns: the ''repeat()'' notation</h4>

	The <dfn>repeat()</dfn> notation represents a repeated fragment of the <a>track list</a>,
	allowing a large number of columns or rows that exhibit a recurring pattern
	to be written in a more compact form.
	The syntax of the ''repeat()'' notation
	has several forms:

	<pre class='prod'>
		<dfn><<track-repeat>></dfn> = repeat( [ <<positive-integer>> ]  , [ <<line-names>>? <<track-size>> ]+ <<line-names>>? )
		<dfn><<auto-repeat>></dfn>  = repeat( [ auto-fill | auto-fit ],   <<line-names>>? <<fixed-size>>    <<line-names>>? )
		<dfn><<fixed-repeat>></dfn> = repeat( [ <<positive-integer>> ]  , [ <<line-names>>? <<fixed-size>> ]+ <<line-names>>? )
		<dfn><<name-repeat>></dfn>  = repeat( [ <<positive-integer>> | auto-fill ], <<line-names>>+)
	</pre>

	The first argument specifies the number of repetitions.
	The second argument is a <a>track list</a>,
	which is repeated that number of times.

	* The <<track-repeat>> variant can represent the repeatition of any <<track-size>>,
		but is limited to fixed repetitions.
	* The <<auto-repeat>> variant can repeat automatically to fill a space,
		but requires <a>definite</a> minimum track sizes so that the number of repetitions can be calculated.
		It can only appear once in the <a>track list</a>,
		but the same <a>track list</a> can also contain <<fixed-repeat>>s.
	* The <<name-repeat>> variant can be used with the ''grid-template-rows/subgrid'' keyword:
		it only repeats names.
		The ''auto-fill'' keyword is only valid once per <<line-name-list>>,
		and repeats enough times for the name list to match the subgrid’s specified <a>grid span</a>
		(defaulting to 0 if that is ''auto'' or if the span is already fulfilled).

	<div class='example'>
		This example shows two equivalent ways of writing the same grid definition.
		Both ways produce a grid with a single row and four "main" columns, each 250px wide,
		surrounded by 10px "gutter" columns.

		<pre>
			&lt;style>
			#grid {
				display: grid;
				grid-template-columns: 10px [col-start] 250px [col-end]
				                       10px [col-start] 250px [col-end]
				                       10px [col-start] 250px [col-end]
				                       10px [col-start] 250px [col-end] 10px;
				grid-template-rows: 1fr;
			}

			/* Equivalent definition. */
			#grid {
				display: grid;
				grid-template-columns: repeat(4, 10px [col-start] 250px [col-end]) 10px;
				grid-template-rows: 1fr;
			}
			&lt;/style>
		</pre>
	</div>

	If the ''repeat()'' function ends up placing two <<line-names>> adjacent to each other,
	the name lists are merged.
	For example, ''repeat(2, [a] 1fr [b])'' is equivalent to ''[a] 1fr [b a] 1fr [b]''.

	When <dfn for="repeat()">''auto-fill''</dfn> is given as the repetition number,
	if the <a>grid container</a> has a <a>definite</a> size or max size in the relevant axis,
	then the number of repetitions is the largest possible positive integer
	that does not cause the <a>grid</a> to overflow its <a>grid container</a>
	(treating each track as its <a>max track sizing function</a> if that is <a>definite</a>
	or as its minimum track sizing function otherwise).
	Otherwise, if the <a>grid container</a> has a <a>definite</a> min size in the relevant axis,
	the number of repetitions is the largest possible positive integer that fulfills that minimum requirement.
	Otherwise, the specified <a>track list</a> repeats only once.

	The <dfn for="repeat()">''auto-fit''</dfn> keyword behaves the same as ''auto-fill'',
	except that after <a href="#auto-placement-algo">grid item placement</a>
	any empty repetitions are dropped.

	Issue: In order to simplify implementation and testing requirements for Level 1,
	the <<auto-repeat>> syntax only accepts a single <<fixed-size>>.
	Now that we have gutters, most use cases for auto-repetition will need only one track size.
	However, if there are important use cases for having a full track listing,
	the CSSWG is not opposed to expanding the syntax;
	please send such use cases to www-style@w3.org
	for consideration.

<!--
████████ ████████        ██     ██ ██    ██ ████ ████████
██       ██     ██       ██     ██ ███   ██  ██     ██
██       ██     ██       ██     ██ ████  ██  ██     ██
██████   ████████        ██     ██ ██ ██ ██  ██     ██
██       ██   ██         ██     ██ ██  ████  ██     ██
██       ██    ██        ██     ██ ██   ███  ██     ██
██       ██     ██        ███████  ██    ██ ████    ██
-->

<h4 id='fr-unit'>
Flexible Lengths: the ''fr'' unit</h4>

	A <dfn export>flexible length</dfn> or <dfn><<flex>></dfn> is a dimension with the ''fr'' unit,
	which represents a fraction of the <a>free space</a> in the <a>grid container</a>.

	The distribution of <a>free space</a> occurs after all non-flexible <a>track sizing functions</a> have reached their maximum.
	The total size of such rows or columns is subtracted from the available space, yielding the <a>free space</a>,
	which is then divided among the flex-sized rows and columns in proportion to their <a>flex factor</a>.

	Note: Flexible lengths in a <a>track list</a> work similarly to flexible lengths with a zero base size in [[CSS3-FLEXBOX]].

	Each column or row's share of the <a>free space</a> can be computed as the column or row's
	<code>&lt;flex> * &lt;free space> / &lt;sum of all <a>flex factors</a>&gt;</code>.
	For the purpose of this calculation,
	a flexible length in the <var>min</var> position of a ''minmax()'' function is treated as ''0'' (an inflexible length).

	When the available space is infinite
	(which happens when the <a>grid container</a>’s width or height is <a>indefinite</a>),
	flex-sized <a>grid tracks</a> are sized to their contents while retaining their respective proportions.
	The used size of each flex-sized <a>grid track</a> is computed by
	determining the ''max-content'' size of each flex-sized <a>grid track</a>
	and dividing that size by the respective <a>flex factor</a>
	to determine a “hypothetical ''1fr'' size”.
	The maximum of those is used as the resolved ''1fr'' length (the <dfn>flex fraction</dfn>),
	which is then multiplied by each <a>grid track</a>’s <a>flex factor</a> to determine its final size.

<!--
 ██████  ██     ██ ████████   ██████   ████████  ████ ████████
██    ██ ██     ██ ██     ██ ██    ██  ██     ██  ██  ██     ██
██       ██     ██ ██     ██ ██        ██     ██  ██  ██     ██
 ██████  ██     ██ ████████  ██   ████ ████████   ██  ██     ██
      ██ ██     ██ ██     ██ ██    ██  ██   ██    ██  ██     ██
██    ██ ██     ██ ██     ██ ██    ██  ██    ██   ██  ██     ██
 ██████   ███████  ████████   ██████   ██     ██ ████ ████████
-->

<h4 id="subgrids">
Subgrids: the ''grid-template-rows/subgrid'' keyword</h4>

	A <a>grid item</a> can itself be a <a>grid container</a> by giving it ''display: grid'';
	in this case the layout of its contents will be independent of the layout of the grid it participates in.

	In some cases it might be necessary for the contents of multiple <a>grid items</a> to align to each other.
	A <a>grid container</a> that is itself a <a>grid item</a>
	can defer the definition of its rows or columns to its parent <a>grid container</a>
	by using the ''grid-template-rows/subgrid'' keyword in 'grid-template-rows' and/or 'grid-template-columns',
	making it a <dfn export>subgrid</dfn> in that dimension.
	In this case, the <a>grid items</a> of the <a>subgrid</a>
	participate in sizing the <a>grid</a> of the parent <a>grid container</a>,
	allowing the contents of both grids to align.

	If a <a>grid container</a> that is <em>not</em> a <a>grid item</a> uses the ''grid-template-rows/subgrid'' keyword,
	its 'grid-template-rows'/'grid-template-columns' value is treated identically to ''grid-template-rows/none''
	(i.e. it doesn't have an <a>explicit grid</a> in that dimension).

	<div class="example">
			For example, suppose we have a form consisting of a list of inputs with labels:
		<pre>
			&lt;ul>
				&lt;li>&lt;label>Name:&lt;/label> &lt;input name=fn>
				&lt;li>&lt;label>Address:&lt;/label> &lt;input name=address>
				&lt;li>&lt;label>Phone:&lt;/label> &lt;input name=phone>
			&lt;/ul>
		</pre>

		We want the labels and inputs to align, and we want to style each list item with a border.
		This can be accomplished with subgrid layout:

		<pre>
			ul {
				display: grid;
				grid-auto-flow: row;
				grid-template-columns: auto 1fr;
			}
			li {
				display: grid;
				grid: subgrid;
				margin: 0.5em;
				border: solid;
				padding: 0.5em;
			}
			label {
				grid-column: 1;
			}
			input {
				grid-column: 2;
			}
		</pre>
	</div>

	A <a>subgrid</a> behaves just like a normal <a>grid container</a> except that:

	<ul style="list-style-type: lower-alpha">
		<li>
			The number of explicit tracks is given by its <a>grid span</a>,
			rather than by 'grid-template-rows'/'grid-template-columns'.
			If the grid has an <a>automatic grid span</a>,
			the number of tracks is determined by the size of the <a>subgrid</a>’s <a>implicit grid</a>.

		<li>
			The <a>grid-placement properties</a> of the <a>subgrid</a>’s <a>grid items</a>
			are scoped to the lines covered by the subgrid.
			E.g., numeric indices count starting from the first line of the subgrid
			rather than the first line of the parent grid.

		<li>
			The <a>subgrid</a>'s own grid items participate in the sizing of its parent grid and are aligned to it.
			In this process, the sum of the <a>subgrid</a>'s margin, padding, and borders are applied as an extra layer of margin to the items at those edges.

			<div class="example">
				For example, if we have a 3&times;3 grid with the following tracks:

				<pre>#parent-grid { grid-template-columns: 300px auto 300px; }</pre>

				If a subgrid covers the last two tracks, its first two columns correspond to the parent grid's last two columns,
				and any items positioned into those tracks participate in sizing the parent grid.
				Specifically, an item positioned in the first track of the subgrid
				influences the auto-sizing of the parent grid's middle track.

				<pre>
					#subgrid { grid-column: 2 / span 2; } /* cover parent's 2nd and 3rd tracks */
					#subgrid :first-child { grid-column: 1; } /* subgrid's 1st track, parent grid's 2nd track */
				</pre>

				If the subgrid has margins/borders/padding,
				the the size of those margins/borders/padding also influence sizing.
				For example, if the subgrid has 100px padding:

				<pre>#subgrid { padding: 100px; }</pre>

				Then when the parent grid auto-sizes its second track,
				it will be at least 100px wider than any items in the subgrid's first track,
				and any items in the subgrid's second track will be sized to fit a slot 200px wide (instead of 300px wide).
			</div>

			However, any overflow tracks
			(i.e. those outside the <a>explicit grid</a> when the <a>subgrid</a> has a <a>definite grid span</a>)
			do not correspond to any tracks in the parent grid;
			they effectively extend in a third dimension.

			<div class="example">
				For example, if a parent grid has adjacent tracks <var>A</var>, <var>B</var>, and <var>C</var>,
				and a ''span 1'' subgrid with an extra <a>implicit grid track</a> is placed in track <var>B</var>,
				the items in that <a>implicit grid track</a> are not considered part of track <var>B</var>.
			</div>

		<li>
			The <a>subgrid</a> is always stretched in its subgridded dimension:
			the 'align-self'/'justify-self' properties on it are ignored in that dimension,
			and any specified width/height constraints are also ignored in that dimension.

		<li>
			Layoutwise, the <a>subgrid</a>’s <a>explicit grid</a> is always aligned with the corresponding section of the parent <a>grid</a>;
			the 'align-content'/'justify-content' properties on it are ignored.
			However, 'overflow' does apply,
			so the contents of the subgrid can be scrolled aside.
			(Note: the act of scrolling does not affect layout.)

		<li>
			Explicit named lines can also be specified together with the ''grid-template-rows/subgrid'' keyword;
			these names apply (within the <a>subgrid</a>) to the corresponding lines of the parent <a>grid</a>.
			If the <a>subgrid</a> has an explicit <a>grid span</a>,
			any names specified for lines beyond the span are ignored.

			<div class="example">
				For example, if the subgrid above were specified with 5 names:

				<pre>#subgrid { grid-template-columns: subgrid [first] [second] [third] [fourth] [fifth]; }</pre>

				Items within the subgrid could be positioned using the first four line names;
				the last name would be ignored (as if it didn't exist),
				since the subgrid only covers four lines.
			</div>

			If the <a>subgrid</a> has an explicit <a>grid position</a> as well as an explicit <a>grid span</a>,
			it also automatically receives the line names specified for its parent <a>grid</a>.
			(In such cases the author can rely on the names specified in the parent grid,
			and does not need to duplicate those names in each subgrid declaration.)

			Issue: People should review how subgrid line names are assigned, to ensure it is sane and useful.
	</ul>

<!--
████████  ████████  ██████   ███████  ██       ██     ██ ████████ ████████
██     ██ ██       ██    ██ ██     ██ ██       ██     ██ ██       ██     ██
██     ██ ██       ██       ██     ██ ██       ██     ██ ██       ██     ██
████████  ██████    ██████  ██     ██ ██       ██     ██ ██████   ██     ██
██   ██   ██             ██ ██     ██ ██        ██   ██  ██       ██     ██
██    ██  ██       ██    ██ ██     ██ ██         ██ ██   ██       ██     ██
██     ██ ████████  ██████   ███████  ████████    ███    ████████ ████████
-->

<h4 id='resolved-track-list'>
Resolved Values</h4>

	When an element's 'display' is ''grid'' or ''inline-grid'' and it generates a box,
	the <a href="http://www.w3.org/TR/cssom/#resolved-values">resolved value</a>
	of the 'grid-template-rows' and 'grid-template-columns' properties is the used value,
	serialized as follows:

	<ul>
		<li>
			Every track listed,
			whether implicitly or explicitly created.

		<li>
			Every track size given as a length in pixels,
			regardless of sizing function.

		<li>
			A contiguous run of two or more tracks that have the same size and associated line names
			may be serialized with the ''repeat()'' notation.
	</ul>

	Otherwise, (e.g. when the element has ''display: none'' or is not a grid)
	the resolved value is simply the computed value.

	<div class="example">
		<pre>
			&lt;style>
			#grid {
				width: 500px;
				grid-template-columns:
				  [a]     auto
				  [b]     minmax(min-content, 1fr)
				  [b c d] repeat(2, [e] 40px)
				          repeat(5, auto);
			}
			&lt;/style>
			&lt;div id="grid">
			  &lt;div style="grid-column-start:1; width:50px">&lt;/div>
			  &lt;div style="grid-column-start:9; width:50px">&lt;/div>
			&lt;/div>
			&lt;script>
			  var gridElement = document.getElementById("grid");
			  getComputedStyle(gridElement).gridTemplateColumns;
			  // [a] 50px [b] 320px [b c d] repeat(2, [e] 40px) repeat(4, 0px) 50px
			&lt;/script>
		</pre>
	</div>

	Note: In general, resolved values are the computed values,
	except for a small list of legacy 2.1 properties.
	However, compatibility with early implementations of this module
	requires us to define 'grid-template-rows' and 'grid-template-columns' as returning used values.
	Authors are recommended to use the
	<a href="http://dev.w3.org/csswg/cssom/#the-getstyleutils-interface"><code>.rawComputedStyle</code> and <code>.usedStyle</code> attributes</a>
	instead of <code>getComputedStyle()</code>.

<!--
 ██████   ████████  ████ ████████           ███    ████████  ████████    ███     ██████
██    ██  ██     ██  ██  ██     ██         ██ ██   ██     ██ ██         ██ ██   ██    ██
██        ██     ██  ██  ██     ██        ██   ██  ██     ██ ██        ██   ██  ██
██   ████ ████████   ██  ██     ██       ██     ██ ████████  ██████   ██     ██  ██████
██    ██  ██   ██    ██  ██     ██       █████████ ██   ██   ██       █████████       ██
██    ██  ██    ██   ██  ██     ██       ██     ██ ██    ██  ██       ██     ██ ██    ██
 ██████   ██     ██ ████ ████████        ██     ██ ██     ██ ████████ ██     ██  ██████
-->

<h3 id="grid-template-areas-property">
Named Areas: the 'grid-template-areas' property</h3>

	<pre class='propdef'>
	Name: grid-template-areas
	Value: none | <<string>>+
	Initial: none
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: n/a
	Media: visual
	Computed value: specified value
	</pre>

	This property specifies <dfn export lt="named grid area">named grid areas</dfn>,
	which are not associated with any particular <a>grid item</a>,
	but can be referenced from the <a>grid-placement properties</a>.
	The syntax of the 'grid-template-areas' property also provides a visualization
	of the structure of the <a>grid</a>,
	making the overall layout of the <a>grid container</a> easier to understand.

	Values have the following meanings:

	<dl dfn-for="grid-template-areas" dfn-type=value>
		<dt><dfn>none</dfn>
		<dd>
			The <a>grid container</a> doesn't define any <a>named grid areas</a>.

		<dt><dfn><<string>>+</dfn>
		<dd>
			A row is created for every separate string listed for the 'grid-template-areas' property,
			and a column is created for each cell in the string,
			when parsed as follows:

			Tokenize the string into a list of the following tokens,
			using longest-match semantics:

			<ul dfn-type="dfn">
				<li>
					A sequence of <a>name code points</a>,
					representing a <dfn>named cell token</dfn>
					with a name consisting of its code points.
				<li>
					A sequence of one or more "." (U+002E FULL STOP),
					representing a <dfn>null cell token</dfn>.

				<li>
					A sequence of <a>whitespace</a>,
					representing nothing
					(do not produce a token).

				<li>
					A sequence of any other characters,
					representing a <dfn>trash token</dfn>.
			</ul>

			Note: These rules can produce cell names that do not match the <<ident>> syntax,
			such as "1st 2nd 3rd",
			which requires escaping when referencing those areas by name in other properties,
			like ''grid-row: \31st;'' to reference the area named ''1st''.

			<ul>
				<li>
					A <a>null cell token</a> represents an unnamed area in the <a>grid container</a>.

				<li>
					A <a>named cell token</a> creates a <a>named grid area</a> with the same name.
					Multiple <a>named cell tokens</a> within and between rows
					create a single <a>named grid area</a> that spans the corresponding <a>grid cells</a>.

				<li>
					A <a>trash token</a> is a syntax error,
					and makes the declaration invalid.
			</ul>

			All strings must have the same number of columns,
			or else the declaration is invalid.
			If a <a>named grid area</a> spans multiple <a>grid cells</a>,
			but those cells do not form a single filled-in rectangle,
			the declaration is invalid.

			Note: Non-rectangular or disconnected regions may be permitted in a future version of this module.
	</dl>


	<div class='example'>
		In this example, the 'grid-template-areas' property is used to create a page layout
		where areas are defined for header content (<code>head</code>),
		navigational content (<code>nav</code>),
		footer content (<code>foot</code>),
		and main content (<code>main</code>).
		Accordingly, the template creates three rows and two columns,
		with four <a>named grid areas</a>.
		The <code>head</code> area spans both columns and the first row of the grid.

		<pre>
			&lt;style type="text/css">
			#grid {
				display: grid;
				grid-template-areas: "head head"
				                     "nav  main"
				                     "foot ...."
			}
			#grid > header { grid-area: head; }
			#grid > nav    { grid-area: nav; }
			#grid > main   { grid-area: main; }
			#grid > footer { grid-area: foot; }
			&lt;/style>
		</pre>
	</div>

<h4 id="implicit-named-lines">
Implicit Named Lines</h4>

	The 'grid-template-areas' property creates <dfn export lt="implicit named line">implicit named lines</dfn> from the <a>named grid areas</a> in the template.
	For each <a>named grid area</a> <var>foo</var>, four <a>implicit named lines</a> are created:
	two named ''<var>foo</var>-start'', naming the row-start and column-start lines of the <a>named grid area</a>,
	and two named ''<var>foo</var>-end'', naming the row-end and column-end lines of the <a>named grid area</a>.

	These named lines behave just like any other named line,
	except that they do not appear in the value of 'grid-template-rows'/'grid-template-columns'.
	Even if an explicit line of the same name is defined,
	the implicit named lines are just more lines with the same name.

<h4 id="implicit-named-areas">
Implicit Named Areas</h4>

	Since a <a>named grid area</a> is referenced by the <a>implicit named lines</a> it produces,
	explicitly adding named lines of the same form (''foo-start''/''foo-end'')
	effectively creates a <a>named grid area</a>.
	Such <dfn export lt="implicit named area">implicit named areas</dfn> do not appear in the value of 'grid-template-areas',
	but can still be referenced by the <a>grid-placement properties</a>.

<!--
 ██████   ████████  ████ ████████          ████████ ████████ ██     ██ ████████  ██          ███    ████████ ████████
██    ██  ██     ██  ██  ██     ██            ██    ██       ███   ███ ██     ██ ██         ██ ██      ██    ██
██        ██     ██  ██  ██     ██            ██    ██       ████ ████ ██     ██ ██        ██   ██     ██    ██
██   ████ ████████   ██  ██     ██ ███████    ██    ██████   ██ ███ ██ ████████  ██       ██     ██    ██    ██████
██    ██  ██   ██    ██  ██     ██            ██    ██       ██     ██ ██        ██       █████████    ██    ██
██    ██  ██    ██   ██  ██     ██            ██    ██       ██     ██ ██        ██       ██     ██    ██    ██
 ██████   ██     ██ ████ ████████             ██    ████████ ██     ██ ██        ████████ ██     ██    ██    ████████
-->

<h3 id='explicit-grid-shorthand'>
Explicit Grid Shorthand: the 'grid-template' property</h3>

	<pre class='propdef'>
	Name: grid-template
	Value: none | subgrid | <<'grid-template-columns'>> / <<'grid-template-rows'>> |<br> [ <<track-list>> / ]? [ <<line-names>>? <<string>> <<track-size>>? <<line-names>>? ]+
	Initial: see individual properties
	Applies to: <a>grid containers</a>
	Inherited: see individual properties
	Percentages: see individual properties
	Media: visual
	Computed value: see individual properties
	</pre>

	The 'grid-template' property is a shorthand for setting
	'grid-template-columns', 'grid-template-rows', and 'grid-template-areas' in a single declaration.
	It has several distinct syntax forms:

	<dl dfn-for=grid-template dfn-type=value>
		<dt><dfn>none</dfn>
		<dd>
			Sets all three properties to their initial values (<a value for=grid-template-rows>none</a>).

		<dt><dfn>subgrid</dfn>
		<dd>
			Sets 'grid-template-rows' and 'grid-template-columns' to ''grid-template-rows/subgrid'',
			and 'grid-template-areas' to its initial value.

		<dt><dfn id='grid-template-rowcol'><<'grid-template-columns'>> / <<'grid-template-rows'>></dfn>
		<dd>
			Sets 'grid-template-columns' and 'grid-template-rows' to the specified values, respectively,
			and sets 'grid-template-areas' to <a value for=grid-template-areas>none</a>.

			<div class='example'>
				<pre>grid-template: auto 1fr auto / auto 1fr;</pre>

					is equivalent to

				<pre>
					grid-template-columns: auto 1fr auto;
					grid-template-rows: auto 1fr;
					grid-template-areas: none;
				</pre>
			</div>

		<dt><dfn id='grid-template-ascii'>[ <<track-list>> / ]? [ <<line-names>>? <<string>> <<track-size>>? <<line-names>>? ]+</dfn>
		<dd>
			<ul>
				<li>
					Sets 'grid-template-columns' to the track listing specified before the slash
					(or <a value for=grid-template-columns>none</a>, if not specified).

				<li>
					Sets 'grid-template-areas' to the strings listed after the slash.

				<li>
					Sets 'grid-template-rows' to the <<track-size>>s following each string
					(filling in <a value for=grid-template-rows>auto</a> for any missing sizes),
					and splicing in the named lines defined before/after each size.
			</ul>

				This syntax allows the author to align track names and sizes inline with their respective grid areas.

			<div class='example'>
				<pre>
					grid-template: auto 1fr auto /
					  [header-top] "a   a   a"     [header-bottom]
					    [main-top] "b   b   b" 1fr [main-bottom];
				</pre>

					is equivalent to

				<pre>
					grid-template-columns: auto 1fr auto;
					grid-template-rows: [header-top] auto [header-bottom main-top] 1fr [main-bottom];
					grid-template-areas: "a a a"
					                     "b b b";
				</pre>
			</div>
	</dl>

	Note: The 'grid' shorthand accepts the same syntax,
	but also resets the implicit grid properties to their initial values.
	Unless authors want those to cascade in separately,
	it is therefore recommended to use 'grid' instead of 'grid-template'.

<!--
████ ██     ██ ████████  ██       ████  ██████  ████ ████████
 ██  ███   ███ ██     ██ ██        ██  ██    ██  ██     ██
 ██  ████ ████ ██     ██ ██        ██  ██        ██     ██
 ██  ██ ███ ██ ████████  ██        ██  ██        ██     ██
 ██  ██     ██ ██        ██        ██  ██        ██     ██
 ██  ██     ██ ██        ██        ██  ██    ██  ██     ██
████ ██     ██ ██        ████████ ████  ██████  ████    ██
-->

<h2 id='implicit-grids'>
The Implicit Grid</h2>

	The 'grid-template' property and its longhands define a fixed number of tracks that form the <a>explicit grid</a>.
	When <a>grid items</a> are positioned outside of these bounds,
	the <a>grid container</a> generates
	<dfn export lt="implicit grid track|implicit grid row|implicit grid column">implicit grid tracks</dfn>
	by adding <dfn>implicit grid lines</dfn> to the <a>grid</a>.
	These lines together with the <a>explicit grid</a> form the <dfn export local-lt="implicit">implicit grid</dfn>.
	The 'grid-auto-rows' and 'grid-auto-columns' properties size these <a>implicit grid tracks</a>.

	The 'grid-auto-flow' property controls auto-placement of <a>grid items</a>
	without an explicit position.
	Once the <a>explicit grid</a> is filled
	(or if there is no <a>explicit grid</a>)
	auto-placement will also cause the generation of <a>implicit grid tracks</a>.

<!--
   ███    ██     ██ ████████  ███████        ████████ ████████     ███     ██████  ██    ██  ██████
  ██ ██   ██     ██    ██    ██     ██          ██    ██     ██   ██ ██   ██    ██ ██   ██  ██    ██
 ██   ██  ██     ██    ██    ██     ██          ██    ██     ██  ██   ██  ██       ██  ██   ██
██     ██ ██     ██    ██    ██     ██          ██    ████████  ██     ██ ██       █████     ██████
█████████ ██     ██    ██    ██     ██          ██    ██   ██   █████████ ██       ██  ██         ██
██     ██ ██     ██    ██    ██     ██          ██    ██    ██  ██     ██ ██    ██ ██   ██  ██    ██
██     ██  ███████     ██     ███████           ██    ██     ██ ██     ██  ██████  ██    ██  ██████
-->

<h3 id='auto-tracks'>
Sizing Auto-generated Rows and Columns: the 'grid-auto-rows' and 'grid-auto-columns' properties</h3>

	<pre class='propdef'>
	Name: grid-auto-columns, grid-auto-rows
	Value: <<track-size>>
	Initial: auto
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: see <a href="#track-sizing">Track Sizing</a>
	Media: visual
	Computed value: see <a href="#track-sizing">Track Sizing</a>
	</pre>

	If a grid item is positioned into a row or column that is not explicitly sized
	by 'grid-template-rows' or 'grid-template-columns',
	<a>implicit grid tracks</a> are created to hold it.
	This can happen either by explicitly positioning into a row or column that is out of range,
	or by the <a>auto-placement algorithm</a> creating additional rows or columns.
	The 'grid-auto-columns' and 'grid-auto-rows' properties specify the size of such implicitly-created tracks.

	<div class='example'>
		This example illustrates the sizing of <a>implicit grid tracks</a>.
		Note that <a>grid item</a> B is positioned on <a>grid line</a> 5,
		which automatically creates <strong>four</strong> <a>implicit grid columns</a>.
		However, only two of them
		(the first and the last)
		are occupied by any <a>grid items</a>,
		so the two empty <a>grid tracks</a>
		collapse to zero width.

		<figure class="figure">
				<img alt="" src="images/implicit-columns-and-rows.png" />

			<figcaption>A Grid with an implicit row and four implicit columns, two of which are zero-sized.</figcaption>
		</figure>

		<pre>
			&lt;style type="text/css">
				#grid {
					display: grid;
					grid-template-columns: 20px;
					grid-template-rows: 20px }
				#A { grid-column: 1;          grid-row: 1; }
				#B { grid-column: 5;          grid-row: 1 / span 2; }
				#C { grid-column: 1 / span 2; grid-row: 2; }
			&lt;/style>

			&lt;div id="grid">
				&lt;div id="A">A&lt;/div>
				&lt;div id="B">B&lt;/div>
				&lt;div id="C">C&lt;/div>
			&lt;/div>
		</pre>
	</div>

<!--
 ██████   ████████  ████ ████████             ███    ██     ██ ████████  ███████          ████████ ██        ███████  ██      ██
██    ██  ██     ██  ██  ██     ██           ██ ██   ██     ██    ██    ██     ██         ██       ██       ██     ██ ██  ██  ██
██        ██     ██  ██  ██     ██          ██   ██  ██     ██    ██    ██     ██         ██       ██       ██     ██ ██  ██  ██
██   ████ ████████   ██  ██     ██ ███████ ██     ██ ██     ██    ██    ██     ██ ███████ ██████   ██       ██     ██ ██  ██  ██
██    ██  ██   ██    ██  ██     ██         █████████ ██     ██    ██    ██     ██         ██       ██       ██     ██ ██  ██  ██
██    ██  ██    ██   ██  ██     ██         ██     ██ ██     ██    ██    ██     ██         ██       ██       ██     ██ ██  ██  ██
 ██████   ██     ██ ████ ████████          ██     ██  ███████     ██     ███████          ██       ████████  ███████   ███  ███
-->

<h3 id="grid-auto-flow-property">
<dfn lt="auto-placement|automatic placement">Automatic Placement</dfn>: the 'grid-auto-flow' property</h3>

	<pre class='propdef'>
	Name: grid-auto-flow
	Value: [ row | column ] || dense
	Initial: row
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: n/a
	Media: visual
	Computed value: specified value
	</pre>

	<a>Grid items</a> that aren't explicitly placed are automatically placed
	into an unoccupied space in the <a>grid container</a>
	by the <a>auto-placement algorithm</a>.
	'grid-auto-flow' controls how the <a>auto-placement algorithm</a> works,
	specifying exactly how auto-placed items get flowed into the grid.
	See [[#auto-placement-algo]] for details on precisely how the auto-placement algorithm works.

	<dl dfn-for="grid-auto-flow" dfn-type=value>
		<dt><dfn>row</dfn>
		<dd>
			The <a>auto-placement algorithm</a> places items
			by filling each row in turn,
			adding new rows as necessary.
			If neither ''row'' nor ''column'' is provided,
			''row'' is assumed.

		<dt><dfn>column</dfn>
		<dd>
			The <a>auto-placement algorithm</a> places items
			by filling each column in turn,
			adding new columns as necessary.

		<dt><dfn>dense</dfn>
		<dd>
			If specified, the <a>auto-placement algorithm</a> uses a “dense” packing algorithm,
			which attempts to fill in holes earlier in the grid if smaller items come up later.
			This may cause items to appear out-of-order,
			when doing so would fill in holes left by larger items.

			If omitted, a “sparse” algorithm is used,
			where the placement algorithm only ever moves “forward” in the grid when placing items,
			never backtracking to fill holes.
			This ensures that all of the auto-placed items appear “in order”,
			even if this leaves holes that could have been filled by later items.
	</dl>

	Note: A future level of this module is expected to add a value that flows auto-positioned items together into a single “default” cell.

	Auto-placement takes <a>grid items</a> in <a>order-modified document order</a>.

	<div class='example'>
		In the following example, there are three columns, each auto-sized to their contents.
		No rows are explicitly defined.
		The 'grid-auto-flow' property is ''grid-auto-flow/row''
		which instructs the grid to search across its three columns starting with the first row,
		then the next,
		adding rows as needed until sufficient space is located to accommodate the position of any auto-placed <a>grid item</a>.

		<div class="figure">
				<img alt="Image: A form arranged using automatic placement." src="images/auto-placed-form.png" />

			<p class="caption">A form arranged using automatic placement.
		</div>

		<pre>
			&lt;style type="text/css">
			form {
				display: grid;
				/* Define three columns, all content-sized,
				   and name the corresponding lines. */
				grid-template-columns: [labels] auto [controls] auto [oversized] auto;
				grid-auto-flow: row dense;
			}
			form > label {
				/* Place all labels in the "labels" column and
				   automatically find the next available row. */
				grid-column: labels;
				grid-row: auto;
			}
			form > input, form > select {
				/* Place all controls in the "controls" column and
				   automatically find the next available row. */
				grid-column: controls;
				grid-row: auto;
			}

			#department-block {
				/* Auto place this item in the "oversized" column
				   in the first row where an area that spans three rows
				   won't overlap other explicitly placed items or areas
				   or any items automatically placed prior to this area. */
				grid-column: oversized;
				grid-row: span 3;
			}

			/* Place all the buttons of the form
			   in the explicitly defined grid area. */
			#buttons {
				grid-row: auto;

				/* Ensure the button area spans the entire grid element
				   in the row axis. */
				grid-column: 1 / -1;
				text-align: end;
			}
			&lt;/style>
			&lt;form>
				&lt;label for="firstname">First name:&lt;/label>
				&lt;input type="text" id="firstname" name="firstname" />
				&lt;label for="lastname">Last name:&lt;/label>
				&lt;input type="text" id="lastname" name="lastname" />
				&lt;label for="address">Address:&lt;/label>
				&lt;input type="text" id="address" name="address" />
				&lt;label for="address2">Address 2:&lt;/label>
				&lt;input type="text" id="address2" name="address2" />
				&lt;label for="city">City:&lt;/label>
				&lt;input type="text" id="city" name="city" />
				&lt;label for="state">State:&lt;/label>
				&lt;select type="text" id="state" name="state">
					&lt;option value="WA">Washington&lt;/option>
				&lt;/select>
				&lt;label for="zip">Zip:&lt;/label>
				&lt;input type="text" id="zip" name="zip" />

				&lt;div id="department-block">
					&lt;label for="department">Department:&lt;/label>
					&lt;select id="department" name="department" multiple>
						&lt;option value="finance">Finance&lt;/option>
						&lt;option value="humanresources">Human Resources&lt;/option>
						&lt;option value="marketing">Marketing&lt;/option>
					&lt;/select>
				&lt;/div>

				&lt;div id="buttons">
					&lt;button id="cancel">Cancel&lt;/button>
					&lt;button id="back">Back&lt;/button>
					&lt;button id="next">Next&lt;/button>
				&lt;/div>
			&lt;/form>
		</pre>
	</div>

<!--
 ██████   ████████  ████ ████████
██    ██  ██     ██  ██  ██     ██
██        ██     ██  ██  ██     ██
██   ████ ████████   ██  ██     ██
██    ██  ██   ██    ██  ██     ██
██    ██  ██    ██   ██  ██     ██
 ██████   ██     ██ ████ ████████
-->

<h2 id='grid-shorthand'>
Grid Definition Shorthand: the 'grid' property</h2>

	<pre class='propdef'>
	Name: grid
	Value: <<'grid-template'>> | [ <<'grid-auto-flow'>> [ <<'grid-auto-columns'>> [ / <<'grid-auto-rows'>> ]? ]? ]
	Initial: see individual properties
	Applies to: <a>grid containers</a>
	Inherited: see individual properties
	Percentages: see individual properties
	Media: visual
	Computed value: see individual properties
	</pre>

	The 'grid' property is a shorthand that sets
	all of the explicit grid properties
	('grid-template-rows', 'grid-template-columns', and 'grid-template-areas'),
	all the implicit grid properties
	('grid-auto-rows', 'grid-auto-columns', and 'grid-auto-flow'),
	and the gutter properties
	('grid-column-gap' and 'grid-row-gap')
	in a single declaration.
	If <<'grid-auto-rows'>> value is omitted,
	it is set to the value specified for 'grid-auto-columns'.
	Other omitted values are set to their initial values.

	Note: Note that you can only specify the explicit <em>or</em> the implicit grid properties in a single 'grid' declaration.
	The sub-properties you don't specify are set to their initial value,
	as normal for shorthands.
	Also, the gutter properties are reset by this shorthand,
	even though they can't be set by it.

	<div class='example'>
		In addition to accepting the 'grid-template' shorthand syntax for setting up the <a>explicit grid</a>,
		the 'grid' shorthand can also easily set up parameters for an auto-formatted grid.
		For example, ''grid: row 1fr;'' is equivalent to

		<pre>
			grid-template: none;
			grid-auto-columns: 1fr;
			grid-auto-rows: 1fr;
			grid-auto-flow: row;
		</pre>

		Similarly, ''grid: column 1fr / auto'' is equivalent to
		<pre>
			grid-template: none;
			grid-auto-columns: 1fr;
			grid-auto-rows: auto;
			grid-auto-flow: column;
		</pre>
	</div>

<!--
 ██████  ██          ███    ██     ██ ████████  ████ ██    ██  ██████
██    ██ ██         ██ ██   ███   ███ ██     ██  ██  ███   ██ ██    ██
██       ██        ██   ██  ████ ████ ██     ██  ██  ████  ██ ██
██       ██       ██     ██ ██ ███ ██ ████████   ██  ██ ██ ██ ██   ████
██       ██       █████████ ██     ██ ██         ██  ██  ████ ██    ██
██    ██ ██       ██     ██ ██     ██ ██         ██  ██   ███ ██    ██
 ██████  ████████ ██     ██ ██     ██ ██        ████ ██    ██  ██████
-->

<h2 id="overlarge-grids">
Clamping Overlarge Grids</h2>

	Since memory is not infinite,
	UAs may clamp the the possible size of the <a>grid</a>
	to within a UA-defined limit,
	dropping all lines outside that limit.
	If a grid item is placed outside this limit,
	its grid area must be <a>clamped</a> to within this limited grid.

	To <dfn local-lt=clamp>clamp a grid area</dfn>:
	* If the <a>grid area</a> would span outside the limited grid,
		its span is clamped to the last line of the limited <a>grid</a>.
	* If the <a>grid area</a> would be placed completely outside the limited grid
		is span must be truncated to 1
		and the area repositioned into the last <a>grid track</a> on that side of the grid.

	<div class='example'>
		For example, if a UA only supported grids with at most 1000 tracks in each dimension,
		the following placement properties:

		<pre class="lang-css">
		.grid-item {
			grid-row: 500 / 1500;
			grid-column: 2000 / 3000;
		}
		</pre>

		Would end up being equivalent to:

		<pre class="lang-css">
		.grid-item {
			grid-row: 500 / 1000;
			grid-column: 999 / 1000;
		}
		</pre>
	</div>

<!--
████████  ██          ███     ██████  ████ ██    ██  ██████
██     ██ ██         ██ ██   ██    ██  ██  ███   ██ ██    ██
██     ██ ██        ██   ██  ██        ██  ████  ██ ██
████████  ██       ██     ██ ██        ██  ██ ██ ██ ██   ████
██        ██       █████████ ██        ██  ██  ████ ██    ██
██        ██       ██     ██ ██    ██  ██  ██   ███ ██    ██
██        ████████ ██     ██  ██████  ████ ██    ██  ██████
-->

<h2 id="placement">
Placing Grid Items</h2>

	Every <a>grid item</a> has a <a>grid area</a>,
	a rectangular set of <a>grid cells</a> that the <a>grid item</a> occupies.
	This <a>grid area</a> defines the <a>containing block</a> for the <a>grid item</a>
	within which the alignment properties ('justify-self' and 'align-self') determine their actual position.
	The cells that a <a>grid item</a> occupies also influence the sizing of the grid's rows and columns,
	defined in [[#layout-algorithm]].

	The location of a <a>grid item’s</a> <a>grid area</a> within the <a>grid</a>
	is defined by its <dfn export lt="grid placement" local-lt="placement">placement</dfn>,
	which consists of a <a>grid position</a> and a <a>grid span</a>:

	<dl export>
		<dt><dfn export local-lt="position|row position|column position">grid position</dfn>
		<dd>
			The <a>grid item</a>’s location in the <a>grid</a>.
			A <a>grid position</a> can be either
			<dfn export lt="definite grid position|definite row position|definite column position|definite position">definite</dfn> (explicitly specified)
			or <dfn export lt="automatic grid position|automatic row position|automatic column position|automatic position">automatic</dfn> (determined by <a>auto-placement</a>).

		<dt><dfn export local-lt="span|row span|column span">grid span</dfn>
		<dd>
			How many <a>grid tracks</a> the <a>grid item</a> occupies.
			A <a>subgrid</a>’s <a>grid span</a> can be <dfn lt="automatic grid span|automatic row span|automatic column span|automatic span">automatic</dfn>
			(determined by its <a>implicit grid</a>) if not specified or implied;
			a <a>grid item</a>’s <a>grid span</a> is otherwise always <dfn export lt="definite grid span|definite row span|definite column span|definite span">definite</dfn> (defaulting to 1).
	</dl>

	The <dfn export lt="grid-placement property">grid-placement properties</dfn>--
	the longhands 'grid-row-start', 'grid-row-end', 'grid-column-start', 'grid-column-end', and their shorthands 'grid-row', 'grid-column', and 'grid-area'--
	allow the author to specify a <a>grid item</a>’s <a>placement</a>
	by providing any (or none) of the following six pieces of information:

	<table class=data style="width: auto">
		<thead>
			<tr>
				<td>
				<th>Row
				<th>Column
		<tbody>
			<tr>
				<th>Start
				<td>row-start line
				<td>column-start line
			<tr>
				<th>End
				<td>row-end line
				<td>column-end line
			<tr>
				<th>Span
				<td>row span
				<td>column span
	</table>

	A definite value for any two of <var>Start</var>, <var>End</var>, and <var>Span</var> in a given dimension implies a definite value for the third.

	The following table summarizes the conditions under which a grid position or span is <i lt>definite</i> or <i lt>automatic</i>:

	<table class='data'>
		<thead>
			<tr>
				<td>
				<th>Position
				<th>Span
		<tbody>
			<tr>
				<th>Definite
				<td>At least one specified line
				<td>Explicit, implicit, or defaulted span. <br> <span class="note">Note: Non-<a>subgrids</a> default to 1.</span>
			<tr>
				<th>Automatic
				<td>No lines explicitly specified
				<td><a>Subgrid</a> without an explicit or implied span
	</table>



<h3 id='common-uses'>
Common Patterns for Grid Placement</h3>

	<em>This section is informative.</em>

	The <a>grid-placement property</a> longhands are organized into three shorthands:

	<table class=data id="grid-property-breakdown">
		<tr>
			<td colspan=4>'grid-area'
		<tr>
			<td colspan=2>'grid-column'
			<td colspan=2>'grid-row'
		<tr>
			<td>'grid-column-start'
			<td>'grid-column-end'
			<td>'grid-row-start'
			<td>'grid-row-end'
	</table>

<h4 id='common-uses-named-areas'>
Named Areas</h4>

	An item can be placed into a <a>named grid area</a>
	(such as those produced by the template in 'grid-template-areas')
	by specifying the area’s name in 'grid-area':

	<div class='example'>
		<pre>
			article {
				grid-area: main;
				/* Places item into the named area "main". */
			}
		</pre>
	</div>

	An item can also be <em>partially</em> aligned with a <a>named grid area</a>,
	with other edges aligned to some other line:

	<div class='example'>
		<pre>
			.one {
				grid-row-start: main;
				/* Align the row-start edge to the start edge of the "main" named area. */
			}
		</pre>
	</div>

<h4 id='common-uses-numeric'>
Numeric Indexes and Spans</h4>

	Grid items can be positioned and sized by number,
	which is particularly helpful for script-driven layouts:

	<div class='example'>
		<pre>
			.two {
				grid-row: 2;    /* Place item in the second row. */
				grid-column: 3; /* Place item in the third column. */
				/* Equivalent to grid-area: 2 / 3;
			}
		</pre>
	</div>

	By default, a grid item has a span of 1.
	Different spans can be given explicitly:

	<div class='example'>
		<pre>
			.three {
				grid-row: 2 / span 5;
				/* Starts in the 2nd row,
				   spans 5 rows down (ending in the 7th row). */
			}

			.four {
				grid-row: span 5 / 7;
				/* <em>Ends</em> in the 7th row,
				   spans 5 rows up (starting in the 2nd row). */
			}
		</pre>
	</div>

	Note: Note that grid indexes are <a>writing mode</a> relative.
	For example, in a right-to-left language like Arabic,
	the first column is the rightmost column.

<h4 id='common-uses-named-lines'>
Named Lines and Spans</h4>

	Instead of counting lines by number,
	<a>named lines</a> can be referenced by their name:

	<div class='example'>
		<pre>
			.five {
				grid-column: first / middle;
				/* Span from line "first" to line "middle". */
			}
		</pre>
	</div>

	Note: Note that if a <a>named grid area</a> and a <a>named line</a> have the same name,
	the placement algorithm will prefer to use <a>named grid area</a>’s edge instead.

	If there are multiple lines of the same name,
	they effectively establish a named set of grid lines,
	which can be exclusively indexed by filtering the placement by name:

	<div class='example'>
		<pre>
			.six {
				grid-row: text 5 / text 7;
				/* Span between the 5th and 7th lines named "text". */
				grid-row: text 5 / span text 2;
				/* Same as above. */
			}
		</pre>
	</div>

<h4 id='common-uses-auto-placement'>
Auto Placement</h4>

	A <a>grid item</a> can be automatically placed into the next available empty <a>grid cell</a>,
	growing the <a>grid</a> if there's no space left.

	<div class='example'>
		<pre>
			.eight {
				grid-area: auto; /* Initial value */
			}
		</pre>
	</div>

	This can be used, for example, to list a number of sale items on a catalog site
	in a grid pattern.

	Auto-placement can be combined with an explicit span,
	if the item should take up more than one cell:

	<div class='example'>
		<pre>
			.nine {
				grid-area: span 2 / span 3;
				/* Auto-placed item, covering two rows and three columns. */
			}
		</pre>
	</div>

	Whether the <a>auto-placement algorithm</a> searchs across and adds rows,
	or searches across and adds columns,
	is controlled by the 'grid-auto-flow' property.

	Note: By default, the <a>auto-placement algorithm</a> looks linearly through the grid without backtracking;
	if it has to skip some empty spaces to place a larger item,
	it will not return to fill those spaces.
	To change this behavior,
	specify the ''dense'' keyword in 'grid-auto-flow'.

<h4 id='common-uses-auto-sizing'>
Auto Sizing Subgrids</h4>

	A subgrid without a <a>definite grid span</a> is sized to its <a>implicit grid</a>,
	i.e. all of its items within it are first placed into its own grid,
	and its span is the resulting size of its grid.

	<div class='example'>
		For example, a subgrid spanning two columns can be given an auto-spanning <a>block size</a>:
		<pre>
			.adverts {
				grid: subgrid;
				grid-column: span 2;
			}
		</pre>

		If it contains 10 auto-placed items, it will span 5 rows in its parent grid.
	</div>

	Note: Since auto-spanning <a>subgrids</a> determine their span before being positioned,
	they can also be auto-positioned.
	The above example would auto-place the <code>.adverts</code> block,
	if no further rules specified its position.

<h3 id="placement-a11y">
Grid Item Placement vs. Source Order</h3>

	<blockquote>
		<p>“With great power comes great responsibility.”
	</blockquote>

	The abilities of the <a>grid-placement properties</a>
	allow content to be freely arranged and reordered within the <a>grid</a>,
	such that the visual presentation can be largely disjoint
	from the underlying document source order.
	These abilities allow the author great freedom
	in tailoring the rendering to different devices
	and modes of presentation
	e.g. using <a>media queries</a>.
	However <strong>they are not a substitute for correct source ordering</strong>.

	Correct source order is important for speech,
	for sequential navigation (such as keyboard navigation),
	and non-CSS UAs such as search engines, tactile browsers, etc.
	Grid placement <em>only</em> affects the visual presentation!
	This allows authors to optimize the document source for
	non-CSS/non-visual interaction modes,
	and use grid placement techniques to further manipulate the visual presentation
	so as to leave that source order intact.


<!--
 ██████   ████████  ████ ████████          ████████   ███████  ██      ██          ██████  ████████    ███    ████████  ████████
██    ██  ██     ██  ██  ██     ██         ██     ██ ██     ██ ██  ██  ██         ██    ██    ██      ██ ██   ██     ██    ██
██        ██     ██  ██  ██     ██         ██     ██ ██     ██ ██  ██  ██         ██          ██     ██   ██  ██     ██    ██
██   ████ ████████   ██  ██     ██ ███████ ████████  ██     ██ ██  ██  ██ ███████  ██████     ██    ██     ██ ████████     ██
██    ██  ██   ██    ██  ██     ██         ██   ██   ██     ██ ██  ██  ██               ██    ██    █████████ ██   ██      ██
██    ██  ██    ██   ██  ██     ██         ██    ██  ██     ██ ██  ██  ██         ██    ██    ██    ██     ██ ██    ██     ██
 ██████   ██     ██ ████ ████████          ██     ██  ███████   ███  ███           ██████     ██    ██     ██ ██     ██    ██
-->

<h3 id='line-placement'>
Line-based Placement: the 'grid-row-start', 'grid-column-start', 'grid-row-end', and 'grid-column-end' properties</h3>

	<pre class='propdef'>
	Name: grid-row-start, grid-column-start, grid-row-end, grid-column-end
	Value: <<grid-line>>
	Initial: auto
	Applies to: <a>grid items</a> and absolutely-positioned boxes whose containing block is a <a>grid container</a>
	Inherited: no
	Percentages: n/a
	Media: visual
	Computed value: specified value
	</pre>

	<pre>
		<dfn for="grid-row-start, grid-column-start, grid-row-end, grid-column-end"><<grid-line>></dfn> =
		  auto |
		  <<custom-ident>> |
		  [ <<integer>> && <<custom-ident>>? ] |
		  [ span &amp;&amp; [ <<integer>> || <<custom-ident>> ] ]
	</pre>

	The 'grid-row-start', 'grid-column-start', 'grid-row-end', and 'grid-column-end' properties
	determine a <a>grid item</a>’s size and location within the <a>grid</a>
	by contributing a line, a span, or nothing (automatic)
	to its <a>grid placement</a>,
	thereby specifying the <a>inline-start</a>, <a>block-start</a>, <a>inline-end</a>, and <a>block-end</a> edges of its <a>grid area</a>.

	Values have the following meanings:

	<dl dfn-for="<grid-line>" dfn-type=value>
		<dt><dfn id='grid-placement-slot'><<custom-ident>></dfn>
		<dd>
			First attempt to match the <a>grid area</a>’s edge to a <a>named grid area</a>:
			if there is a <a>named line</a> with the name ''<<custom-ident>>-start'' (for 'grid-*-start') / ''<<custom-ident>>-end'' (for 'grid-*-end'),
			contributes the first such line to the <a>grid item</a>’s <a>placement</a>.

			Note: <a>Named grid areas</a> automatically generate <a>implicit named lines</a> of this form,
			so specifying ''grid-row-start: foo'' will choose the start edge of that <a>named grid area</a>
			(unless another line named ''foo-start'' was explicitly specified before it).

			Otherwise,
			treat this as if the integer ''1'' had been specified along with the <<custom-ident>>.

		<dt><dfn id='grid-placement-int'><<integer>> && <<custom-ident>>?</dfn>
		<dd>
			Contributes the <var>N</var>th <a>grid line</a> to the <a>grid item</a>’s <a>placement</a>.
			If a negative integer is given,
			it instead counts in reverse,
			starting from the end edge of the <a>explicit grid</a>.

			If a name is given as a <<custom-ident>>,
			only lines with that name are counted.
			If not enough lines with that name exist,
			all <a>implicit grid lines</a> are assumed to have that name for the purpose of finding this position.

			A <<integer>> value of zero makes the declaration invalid.

		<dt><dfn id='grid-placement-span-int'>span && [ <<integer>> || <<custom-ident>> ]</var></dfn>
		<dd>
			Contributes a <a>grid span</a> to the <a>grid item</a>’s <a>placement</a>
			such that the corresponding edge of the <a>grid item</a>’s <a>grid area</a> is <var>N</var> lines from its opposite edge.

			If a name is given as a <<custom-ident>>,
			only lines with that name are counted.
			If not enough lines with that name exist,
			all <a>implicit grid lines</a> are assumed to have that name for the purpose of counting this span.

			If the <<integer>> is omitted, it defaults to ''1''.
			Negative integers or zero are invalid.

		<dt><dfn id='grid-placement-auto'>auto</dfn>
		<dd>
			The property contributes nothing to the <a>grid item</a>’s <a>placement</a>,
			indicating <a>auto-placement</a>, an <a>automatic span</a>, or a default span of one.
			(See [[#placement]], above.)
	</dl>

	<div class='example'>
			Given a single-row, 8-column grid and the following 9 named lines:

		<pre>
			1  2  3  4  5  6  7  8  9
			+--+--+--+--+--+--+--+--+
			|  |  |  |  |  |  |  |  |
			A  B  C  A  B  C  A  B  C
			|  |  |  |  |  |  |  |  |
			+--+--+--+--+--+--+--+--+
		</pre>

			The following declarations place the grid item between the lines indicated by index:

		<pre>
			grid-column-start: 4; grid-column-end: auto;
			/* Line 4 to line 5 */

			grid-column-start: auto; grid-column-end: 6;
			/* Line 5 to line 6 */

			grid-column-start: C; grid-column-end: C -1;
			/* Line 3 to line 9 */

			grid-column-start: C; grid-column-end: span C;
			/* Line 3 to line 6 */

			grid-column-start: span C; grid-column-end: C -1;
			/* Line 6 to line 9 */

			grid-column-start: span C; grid-column-end: span C;
			/* Error: The end span is ignored, and an auto-placed
			   item can't span to a named line.
			   Equivalent to ''grid-column: span 1;''. */

			grid-column-start: 5; grid-column-end: C -1;
			/* Line 5 to line 9 */

			grid-column-start: 5; grid-column-end: span C;
			/* Line 5 to line 6 */

			grid-column-start: 8; grid-column-end: 8;
			/* Error: line 8 to line 9 */

			grid-column-start: B 2; grid-column-end: span 1;
			/* Line 5 to line 6 */
		</pre>
	</div>

<h4 id='grid-placement-errors'>
Grid Placement Conflict Handling</h4>

	If the <a>placement</a> for a <a>grid item</a> contains two lines,
	and the <a>start</a> line is further end-ward than the <a>end</a> line,
	swap the two lines.

	If the <a>placement</a> contains two spans,
	remove the one contributed by the <a>end</a> <a>grid-placement property</a>.

	If the <a>placement</a> contains only a span for a named line,
	replace it with a span of 1.

<!--
 ██████   ████████  ████ ████████           ██████   ███████  ██       ██     ██ ██     ██ ██    ██
██    ██  ██     ██  ██  ██     ██         ██    ██ ██     ██ ██       ██     ██ ███   ███ ███   ██
██        ██     ██  ██  ██     ██         ██       ██     ██ ██       ██     ██ ████ ████ ████  ██
██   ████ ████████   ██  ██     ██ ███████ ██       ██     ██ ██       ██     ██ ██ ███ ██ ██ ██ ██
██    ██  ██   ██    ██  ██     ██         ██       ██     ██ ██       ██     ██ ██     ██ ██  ████
██    ██  ██    ██   ██  ██     ██         ██    ██ ██     ██ ██       ██     ██ ██     ██ ██   ███
 ██████   ██     ██ ████ ████████           ██████   ███████  ████████  ███████  ██     ██ ██    ██
-->

<h3 id='placement-shorthands'>
Placement Shorthands: the 'grid-column', 'grid-row', and 'grid-area' properties</h3>

	<pre class='propdef'>
	Name: grid-row, grid-column
	Value: <<grid-line>> [ / <<grid-line>> ]?
	Initial: see individual properties
	Applies to: <a>grid items</a> and absolutely-positioned boxes whose containing block is a <a>grid container</a>
	Inherited: see individual properties
	Percentages: see individual properties
	Media: visual
	Computed value: see individual properties
	</pre>

	The 'grid-row' and 'grid-column' properties are shorthands for 'grid-row-start'/'grid-row-end' and 'grid-column-start'/'grid-column-end', respectively.

	If two <<grid-line>> values are specified,
	the 'grid-row-start'/'grid-column-start' longhand is set to the value before the slash,
	and the 'grid-row-end'/'grid-column-end' longhand is set to the value after the slash.

	When the second value is omitted,
	if the first value is a <<custom-ident>>,
	the 'grid-row-end'/'grid-column-end' longhand is also set to that <<custom-ident>>;
	otherwise, it is set to <a value for="<grid-line>">auto</a>.

	<pre class='propdef'>
	Name: grid-area
	Value: <<grid-line>> [ / <<grid-line>> ]{0,3}
	Initial: see individual properties
	Applies to: <a>grid items</a> and absolutely-positioned boxes whose containing block is a <a>grid container</a>
	Inherited: see individual properties
	Percentages: see individual properties
	Media: visual
	Computed value: see individual properties
	</pre>

	If four <<grid-line>> values are specified,
	'grid-row-start' is set to the first value,
	'grid-column-start' is set to the second value,
	'grid-row-end' is set to the third value,
	and 'grid-column-end' is set to the fourth value.

	When 'grid-column-end' is omitted,
	if 'grid-column-start' is a <<custom-ident>>,
	'grid-column-end' is set to that <<custom-ident>>;
	otherwise, it is set to <a value for="<grid-line>">auto</a>.

	When 'grid-row-end' is omitted,
	if 'grid-row-start' is a <<custom-ident>>,
	'grid-row-end' is set to that <<custom-ident>>;
	otherwise, it is set to <a value for="<grid-line>">auto</a>.

	When 'grid-column-start' is omitted,
	if 'grid-row-start' is a <<custom-ident>>,
	all four longhands are set to that value.
	Otherwise, it is set to <a value for="<grid-line>">auto</a>.

	Note: The resolution order for this shorthand is row-start/column-start/row-end/column-end,
	which goes CCW for LTR pages,
	the opposite direction of the related 4-edge properties using physical directions, like 'margin'.

<!--
   ███    ██     ██ ████████  ███████          ████████  ██          ███     ██████  ████████ ██     ██ ████████ ██    ██ ████████
  ██ ██   ██     ██    ██    ██     ██         ██     ██ ██         ██ ██   ██    ██ ██       ███   ███ ██       ███   ██    ██
 ██   ██  ██     ██    ██    ██     ██         ██     ██ ██        ██   ██  ██       ██       ████ ████ ██       ████  ██    ██
██     ██ ██     ██    ██    ██     ██ ███████ ████████  ██       ██     ██ ██       ██████   ██ ███ ██ ██████   ██ ██ ██    ██
█████████ ██     ██    ██    ██     ██         ██        ██       █████████ ██       ██       ██     ██ ██       ██  ████    ██
██     ██ ██     ██    ██    ██     ██         ██        ██       ██     ██ ██    ██ ██       ██     ██ ██       ██   ███    ██
██     ██  ███████     ██     ███████          ██        ████████ ██     ██  ██████  ████████ ██     ██ ████████ ██    ██    ██
-->

<h3 id='auto-placement-algo'>
Grid Item Placement Algorithm</h3>

	The following <dfn export local-lt="auto-placement algorithm">grid item placement algorithm</dfn>
	resolves <a>automatic positions</a> of <a>grid items</a> into <a>definite positions</a>,
	ensuring that every <a>grid item</a> has a well-defined <a>grid area</a> to lay out into.
	(<a>Grid spans</a> need no special resolution;
		if they're not explicitly specified,
		they default to either 1 or,
		for <a>subgrids</a>,
		the size of their <a>explicit grid</a>.)

	Note: This algorithm can result in the creation of new rows or columns in the <a>implicit grid</a>,
	if there is no room in the <a>explicit grid</a> to place an auto-positioned <a>grid item</a>.

	Every <a>grid cell</a>
	(in both the <a>explicit</a> and <a>implicit grids</a>)
	can be <dfn>occupied</dfn> or <dfn>unoccupied</dfn>.
	A cell is <a>occupied</a>
	if it's covered by a <a>named grid area</a>,
	or by the <a>grid area</a> of a <a>grid item</a> with a <a>definite grid position</a>;
	otherwise,
	the cell is <a>unoccupied</a>.
	A cell's <a>occupied</a>/<a>unoccupied</a> status can change during this algorithm.

	To aid in clarity,
	this algorithm is written with the assumption that 'grid-auto-flow'
	has ''grid-auto-flow/row'' specified.
	If it is instead set to ''grid-auto-flow/column'',
	swap all mentions of rows and columns, inline and block, etc. in this algorithm.

	1. <strong>Position anything that's not auto-positioned.</strong>

	2. <strong>Process the items locked to a given row.</strong>

		For each <a>grid item</a> with a <a>definite row position</a>
		(that is, the 'grid-row-start' and 'grid-row-end' properties define a <a>definite grid position</a>),
		in <a>order-modified document order</a>:

		: “sparse” packing (omitted ''dense'' keyword)
		::
			Set the column-start line of its <a>placement</a>
			to the earliest (smallest positive index) line index
			that ensures this item’s <a>grid area</a> will not overlap any <a>occupied</a> grid cells
			and that is past any <a>grid items</a> previously placed in this row by this step.

		: “dense” packing (''dense'' specified)
		::
			Set the column-start line of its <a>placement</a>
			to the earliest (smallest positive index) line index
			that ensures this item’s <a>grid area</a> will not overlap any <a>occupied</a> grid cells.

	3. <strong>Determine the number of columns in the implicit grid.</strong>

		Set the number of columns in the <a>implicit grid</a> to the larger of:

		* The number of columns in the <a>explicit grid</a>.
		* Among all the items with a <a>definite column position</a>
			(explicitly positioned items, items positioned in the previous step, and items not yet positioned but with a definite column)
			the largest positive column-end line index,
			minus 1.
		* The largest <a>column span</a> of all items.

		<div class='example'>
			For example, in the following style fragment:

			<pre>
				#grid {
					display: grid;
					grid-template-columns: repeat(5, 100px);
					grid-auto-flow: row;
				}
				#grid-item {
					grid-column: 4 / span 3;
				}
			</pre>

			The number of columns needed is 6.
			The <code>#grid-item</code> element’s column-start line is index 4,
			so its span ensures that it’s column-end line will be index 7.
			This requires <code>7 - 1 = 6</code> columns to hold,
			which is larger than the <a>explicit grid</a>’s 5 columns.
		</div>

	4. <strong>Position the remaining grid items.</strong>

		The <dfn export local-lt="cursor">auto-placement cursor</dfn> defines the current “insertion point” in the grid,
		specified as a pair of row and column <a>grid lines</a>.
		Initially the <a>auto-placement cursor</a> is specified with a row and column position both equal to 1.

		The 'grid-auto-flow' value in use determines how to position the items:

		: “sparse” packing (omitted ''dense'' keyword)
		:: For each <a>grid item</a> that hasn't been positioned by the previous steps,
			in <a>order-modified document order</a>:

			: If the item has a <a>definite column position</a>:
			::
				1. Set the column position of the <a>cursor</a> to be equal to the column-start line index of the <a>grid item</a>.
					If this is less than the previous column position of the <a>cursor</a>,
					increment the row position by 1.
				2. Increment the <a>cursor</a>’s row position until a value is found
					where the <a>grid item</a> does not overlap any <a>occupied</a> grid cells
					(creating new rows in the <a>implicit grid</a> as necessary).
				3. Set the item's row-start line to the <a>cursor's</a> row position.
					(Implicitly setting the item's row-end line according to its span, as well.)

			: If the item has an <a>automatic grid position</a> in both axes:
			::
				1. Increment the column position of the <a>auto-placement cursor</a>
					until either this item’s <a>grid area</a> does not overlap any <a>occupied</a> grid cells,
					or the <a>cursor's</a> column position,
					plus the item's column span,
					overflow the number of columns in the implicit grid,
					as determined earlier in this algorithm.
				2. If a non-overlapping position was found in the previous step,
					set the item's row-start and column-start lines to the <a>cursor's</a> position.
					Otherwise,
					increment the <a>auto-placement cursor</a>’s row position
					(creating new rows in the <a>implicit grid</a> as necessary),
					set its column position to 1,
					and return to the previous step.

		: “dense” packing (''dense'' specified)
		:: For each <a>grid item</a> that hasn't been positioned by the previous steps,
			in <a>order-modified document order</a>:

			: If the item has a <a>definite column position</a>:
			::
				1. Set the row position of the cursor to 1.
					Set the column position of the cursor to be equal to the column-start line index of the <a>grid item</a>.
				2. Increment the <a>auto-placement cursor</a>’s row position until a value is found
					where the <a>grid item</a> does not overlap any <a>occupied</a> grid cells
					(creating new rows in the <a>implicit grid</a> as necessary).
				3. Set the item's row-start line index to the <a>cursor's</a> row position.
					(Implicitly setting the item's row-end line according to its span, as well.)

			: If the item has an <a>automatic grid position</a> in both axes:
			::
				1. Set the cursor's row and column positions both to 1.
				2. Increment the column position of the <a>auto-placement cursor</a>
					until either this item’s <a>grid area</a> does not overlap any <a>occupied</a> grid cells,
					or the <a>cursor's</a> column position,
					plus the item's column span,
					overflow the number of columns in the implicit grid,
					as determined earlier in this algorithm.
				3. If a non-overlapping position was found in the previous step,
					set the item's row-start and column-start lines to the <a>cursor's</a> position.
					Otherwise,
					increment the <a>auto-placement cursor</a>’s row position
					(creating new rows in the <a>implicit grid</a> as necessary),
					set its column position to 1,
					and return to the previous step.

<!--
   ███    ████████   ██████  ████████   ███████   ██████
  ██ ██   ██     ██ ██    ██ ██     ██ ██     ██ ██    ██
 ██   ██  ██     ██ ██       ██     ██ ██     ██ ██
██     ██ ████████   ██████  ████████  ██     ██  ██████
█████████ ██     ██       ██ ██        ██     ██       ██
██     ██ ██     ██ ██    ██ ██        ██     ██ ██    ██
██     ██ ████████   ██████  ██         ███████   ██████
-->

<h2 id="abspos">
Absolute Positioning</h2>

<h3 id="abspos-items">
With a Grid Container as Containing Block</h3>

	If an absolutely positioned element's <a>containing block</a>
	is generated by a <a>grid container</a>,
	the containing block corresponds to the <a>grid area</a> determined by its <a>grid-placement properties</a>.
	The offset properties ('top'/'right'/'bottom'/'left')
	then indicate offsets inwards from the corresponding edges
	of this <a>containing block</a>, as normal.

	Note: While absolutely-positioning an element to a <a>grid container</a>
	does allow it to align to that container's <a>grid lines</a>,
	such elements do not take up space or otherwise participate in the layout of the grid.

	<div class='example'>
		<pre>
			.grid {
				grid: 10rem 10rem 10rem 10rem / 1fr 1fr 1fr 1fr;
				/* 4 columns of ''10rem'' each,
				   4 equal-height rows filling the <a>grid container</a> */
				justify-content: center;
				/* center the grid horizontally within the <a>grid container</a> */
				position: relative;
				/* Establish abspos <a>containing block</a> */
			}

			.abspos {
				grid-row-start: 1;     /* 1st grid row line = top of grid container */
				grid-row-end: span 2;  /* 3rd grid row line */
				grid-column-start: 3;  /* 3rd grid col line */
				grid-column-end: auto; /* right padding edge */
				/* <a>Containing block</a> covers the top right quadrant of the <a>grid container</a> */

				position: absolute;
				top: 70px;
				bottom: 40px;
				left: 100px;
				right: 30px;
			}
		</pre>

		<figure>
			<img src="images/abspos-grid.svg" width="702" height="402" alt=''>
		</figure>

		Note: Grids and the <a>grid-placement properties</a> are <a>flow-relative</a>,
		while abspos offsets are <a>physical</a>,
		so if the 'direction' or 'writing-mode' properties change,
		the grid will transform to match,
		but the offsets won't.
	</div>

	Instead of auto-placement, an ''auto'' value for a <a>grid-placement property</a>
	contributes a special line to the <a>placement</a>
	whose position is that of the corresponding padding edge of the <a>grid container</a>
	(the padding edge of the scrollable area, if the <a>grid container</a> overflows).
	These lines become the first and last lines (0th and -0th) of the <dfn>augmented grid</dfn>
	used for positioning absolutely-positioned items.

	Note: Thus, by default, the absolutely-positioned box's <a>containing block</a> will correspond
	to the padding edges of the <a>grid container</a>, as it does for <a>block containers</a>.

	Absolute positioning occurs after layout of the <a>grid</a>	and its in-flow contents
	and does not contribute to the sizing of any grid tracks
	or affect the size/configuration of the grid in any way.
	If a <a>grid-placement property</a> refers to a non-existent line
	either by explicitly specifying such a line or by spanning outside of the existing <a>implicit grid</a>,
	it is instead treated as specifying ''auto''
	(instead of creating new <a>implicit grid lines</a>).

	If the <a>placement</a> only contains a <a>grid span</a>,
	replace it with the two ''auto'' lines in that axis.
	(This happens when both <a>grid-placement properties</a> in an axis contributed a span originally,
	and [[#grid-placement-errors]] caused the second span to be ignored.)


<h3 id="static-position">
With a Grid Container as Parent</h3>

	The <a href="http://www.w3.org/TR/CSS21/visudet.html#abs-non-replaced-width">static position</a> [[!CSS21]]
	of an absolutely-positioned child of a <a>grid container</a>
	is determined as if it were the sole grid item
	in a <a>grid area</a>
	whose edges coincide with the padding edges of the <a>grid container</a>.
	However, if the <a>grid container</a> parent is also the generator of the absolutely positioned element's <a>containing block</a>,
	instead use the <a>grid area</a> determined in [[#abspos-items]].

	Note: Note that this position is affected by the values of 'justify-self' and 'align-self' on the child,
	and that, as in most other layout models,
	the absolutely-positioned child has no effect on the size of the containing block
	or layout of its contents.

<!--
   ███    ██       ████  ██████   ██    ██ ██     ██ ████████ ██    ██ ████████
  ██ ██   ██        ██  ██    ██  ███   ██ ███   ███ ██       ███   ██    ██
 ██   ██  ██        ██  ██        ████  ██ ████ ████ ██       ████  ██    ██
██     ██ ██        ██  ██   ████ ██ ██ ██ ██ ███ ██ ██████   ██ ██ ██    ██
█████████ ██        ██  ██    ██  ██  ████ ██     ██ ██       ██  ████    ██
██     ██ ██        ██  ██    ██  ██   ███ ██     ██ ██       ██   ███    ██
██     ██ ████████ ████  ██████   ██    ██ ██     ██ ████████ ██    ██    ██
-->

<h2 id='alignment'>
Alignment and Spacing</h2>

	After a <a>grid container</a>’s <a>grid tracks</a> have been sized,
	and the dimensions of all <a>grid items</a> are finalized,
	<a>grid items</a> can be aligned within their <a>grid areas</a>.

	The 'margin' properties can be used to align items in a manner similar to,
	but more powerful than,
	what margins can do in block layout.
	<a>Grid items</a> also respect the alignment properties from the Box Alignment spec,
	which allow easy keyword-based alignment of items in both the <a>row axis</a> and <a>column axis</a>.

	By default,
	<a>grid items</a> stretch to fill their <a>grid area</a>.
	However, if 'justify-self' or 'align-self' compute to a value other than ''stretch''
	or margins are <a value for="margin">auto</a>,
	<a>grid items</a> will auto-size to fit their contents.

<h3 id='gutters'>
Gutters: the 'grid-column-gap', 'grid-row-gap', and 'grid-gap' properties</h3>

	<pre class='propdef'>
	Name: grid-row-gap, grid-column-gap
	Value: <<length>>
	Initial: 0
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: n/a
	Media: visual
	Computed value: specified value
	</pre>

	These properties specify the <dfn lt=gutter>gutters</dfn> between
	<a>grid rows</a> and <a>grid columns</a>, respectively.
	The effect is as though the affected <a>grid lines</a> acquired width:
	the <a>grid track</a> between two <a>grid lines</a>
	is the space between the <i>gutters</i> that represent them.
	For the purpose of <a href="#algo-track-sizing">track sizing</a>,
	each <i>gutter</i> is essentially treated as an extra track of the specified length.

	Note: Additional spacing may be added between tracks
	due to 'justify-content'/'align-content'.
	See [[#algo-overview]].
	This space effectively increases the size of the <i>gutters</i>.

	<pre class='propdef'>
	Name: grid-gap
	Value: <<'grid-column-gap'>> <<'grid-row-gap'>>?
	Initial: normal
	Applies to: <a>grid containers</a>
	Inherited: no
	Percentages: n/a
	Media: visual
	Computed value: specified value
	</pre>

	This property is a <a>shorthand</a> that sets 'grid-column-gap' and 'grid-row-gap' in one declaration.
	If <<'grid-row-gap'>> is omitted,
	it's set to the same value as <<'grid-column-gap'>>.

<h3 id='auto-margins'>
Aligning with <a value for="margin">auto</a> margins</h3>

	<em>This section is non-normative.
		The normative definition of how margins affect grid items is in <a section href="#layout-algorithm"></a>.</em>

	Auto margins on <a>grid items</a> have an effect very similar to auto margins in block flow:

	<ul>
		<li>
			During calculations of <a>grid track</a> sizes, auto margins are treated as ''0''.

		<li>
			Auto margins absorb positive free space
			prior to alignment via the alignment properties
			(defined in the next sections).

		<li>
			Overflowing elements ignore their auto margins and overflow in the <a>end</a> directions.
	</ul>

	Note: Note that, if free space is distributed to auto margins,
	the alignment properties will have no effect in that dimension
	because the margins will have stolen all the free space
	left over after sizing.

	<div class='example'>
		<p class='issue'>
			TODO: EXAMPLE HERE
	</div>


<h3 id='row-align'>
Row-axis Alignment: the 'justify-self' and 'justify-items' properties</h3>

	<a>Grid items</a> can be aligned in the inline dimension
	by using the 'justify-self' property on the <a>grid item</a>
	or 'justify-items' property on the <a>grid container</a>,
	as defined in [[!CSS3-ALIGN]].

	<div class='example'>
		For example,
		for an English document,
		the inline axis is horizontal,
		and so the 'justify-*' properties align the <a>grid items</a> horizontally.

		<p class='issue'>
			TODO: provide example
	</div>


<h3 id='column-align'>
Column-axis Alignment: the 'align-self' and 'align-items' properties</h3>

	<a>Grid items</a> can also be aligned in the block dimension
	(perpendicular to the inline dimension)
	by using the 'align-self' property on the <a>grid item</a>
	or 'align-items' property on the <a>grid container</a>,
	as defined in [[!CSS3-ALIGN]].

<h3 id='grid-align'>
Aligning the Grid: the 'justify-content' and 'align-content' properties</h3>

	If the <a>grid</a>’s outer edges do not correspond to the <a>grid container</a>’s content edges
	(for example, if no columns are flex-sized),
	the <a>grid tracks</a> are aligned within the content box according to the 'justify-content' and 'align-content' properties on the <a>grid container</a>.

	<div class='example'>
		For example, the following grid is centered vertically,
		and aligned to the right edge of its <a>grid container</a>:

		<pre>
			.grid {
				display: grid;
				grid: 12rem 12rem 12rem 12rem / 10rem 10rem 10rem 10rem;
				justify-content: end;
				align-content: center;
			}
		</pre>

		<figure>
			<img src="images/align-justify-content.svg" width="352" height="252" alt=''>
		</figure>
	</div>

	Note that certain values of 'justify-content' and 'align-content'
	can cause the tracks to be spaced apart
	(''justify-content/space-around'', ''justify-content/space-between'', ''justify-content/space-evenly'')
	or to be resized (''stretch'').

	Issue: Add example of fixed size spanner getting extra space from content-distribution.

<h3 id='grid-baselines'>
Grid Baselines</h3>

	The baselines of a <a>grid container</a> are determined as follows:

	<ol>
		<li>
			If any of the <a>grid items</a> whose areas intersect the <a>grid container</a>’s first row/column
			<a href="#baseline-participation">participate in baseline alignment</a>,
			the grid container's baseline is the baseline of those <a>grid items</a>.

		<li>
			Otherwise, if the grid container has at least one <a>grid item</a> whose area intersects the first row/column,
			and the first such <a>grid item</a> (in <a>order-modified grid order</a>) has a baseline
			parallel to the relevant axis,
			the grid container's baseline is that baseline.

		<li>
			Otherwise, the grid container's baseline is <a href="http://www.w3.org/TR/css3-writing-modes/#inline-alignment">synthesized</a>
			from the first item's (in <a>order-modified grid order</a>) content box,
			or, failing that, from the grid container's content box.
	</ol>

	A <a>grid item</a> <dfn export id="baseline-participation">participates in baseline alignment</dfn> in a particular dimension
	if its value for 'align-self' or 'justify-self', as appropriate, is ''baseline''
	and its inline axis is parallel to that dimension.

	<dfn export lt="order-modified grid order">'order'-modified grid order</dfn> is the order in which
	<a>grid items</a> are encountered when traversing the grid's <a>grid cells</a>,
	in row-major order if calculating the inline-axis baseline,
	or in column-major order if calculating the block-axis baseline.
	If two items are encountered at the same time,
	they are taken in <a>order-modified document order</a>.

	When calculating the baseline according to the above rules,
	if the box contributing a baseline has an 'overflow' value that allows scrolling,
	the box must be treated as being in its initial scroll position
	for the purpose of determining its baseline.

	When <a href="http://www.w3.org/TR/CSS21/tables.html#height-layout">determining the baseline of a table cell</a>,
	a grid container provides a baseline just as a line box or table-row does. [[!CSS21]]

<!--
 ██████  ████ ████████ ████ ██    ██  ██████
██    ██  ██       ██   ██  ███   ██ ██    ██
██        ██      ██    ██  ████  ██ ██
 ██████   ██     ██     ██  ██ ██ ██ ██   ████
      ██  ██    ██      ██  ██  ████ ██    ██
██    ██  ██   ██       ██  ██   ███ ██    ██
 ██████  ████ ████████ ████ ██    ██  ██████
-->

<h2 id='layout-algorithm'>
Grid Sizing</h2>

	This section defines the <dfn export>grid sizing algorithm</dfn>,
	which determines the size of all <a>grid tracks</a>
	and, by extension, the entire grid.

	Each track has specified <a lt="min track sizing function">minimum</a> and
	<a lt="max track sizing function">maximum</a> <a>sizing functions</a>
	(which may be the same).
	Each <a>sizing function</a> is either:

	<ul>
		<li>A <dfn>fixed sizing function</dfn> (<<length>> or resolveable <<percentage>>).
		<li>An <dfn>intrinsic sizing function</dfn> (''min-content'', ''max-content'', or ''auto'').
		<li>A <dfn>flexible sizing function</dfn> (<<flex>>).
	</ul>

	The <a>grid sizing algorithm</a> defines how to resolve these sizing constraints into used track sizes.

<h3 id="algo-overview">
Grid Sizing Algorithm</h3>

	<ol>
		<li>
			First, the <a>track sizing algorithm</a> is used to resolve the sizes of the <a>grid columns</a>.

			If calculating the layout of a <a>grid item</a> in this step
			depends on the <a>available space</a> in the <a>row axis</a>,
			assume the <a>available space</a> that it would have
			if any row with a <a>definite</a> <a>max track sizing function</a>
			had that size and all other rows were infinite.

		<li>
			Next, the <a>track sizing algorithm</a> resolves the sizes of the <a>grid rows</a>,
			using the <a>grid column</a> sizes calculated in the previous step.

		<li>
			Then, if the <a>min-content contribution</a> of any grid items have changed
			based on the row sizes calculated in step 2,
			steps 1 and 2 are repeated with the new <a>min-content contribution</a> (once only).

			<div class="note">
				This cycle is necessary for cases where the <a>inline size</a> of a <a>grid item</a>
				depends on the <a>block size</a> of its <a>grid area</a>.
				Examples include wrapped column <a>flex containers</a> (''flex-flow: column wrap''),
				<a>orthogonal flows</a> ('writing-mode'),
				and <a spec=css-multicol>multi-column elements</a>.
			</div>

			Issue: Is one cycle enough?

		<li>
			Finally, the <a>grid container</a> is sized
			using the resulting size of the <a>grid</a> as its content size,
			and the tracks are aligned within the <a>grid container</a>
			according to the 'align-content' and 'justify-content' properties.
			<span class="note">Note: This can introduce extra space within or between tracks.</span>
			When introducing space within tracks,
			only tracks with an ''grid-template-rows/auto'' <a>max track sizing function</a> accept space.
	</ol>

	Once the size of each <a>grid area</a> is thus established,
	the <a>grid items</a> are laid out into their respective containing blocks.

<h3 id='algo-terms'>
Track Sizing Terminology</h3>

	<dl>
		<dt><dfn>min track sizing function</dfn>
		<dd>
			If the track was sized with a ''minmax()'' function,
			this is the first argument to that function.
			Otherwise, it's the track's sizing function.

		<dt><dfn>max track sizing function</dfn>
		<dd>
			If the track was sized with a ''minmax()'' function,
			this is the second argument to that function.
			Otherwise, it's the track's sizing function.

		<dt><dfn>free space</dfn>
		<dd>
			Equal to the <a>available space</a> minus the sum of the <a>base sizes</a> of all the grid tracks,
			clamped to a minimum of zero.
			This value can change over the course of the algorithm,
			as <a>base sizes</a> are adjusted.

			If <a>available space</a> is <a>indefinite</a>,
			the <a>free space</a> is <a>indefinite</a> as well.

		<dt><dfn>span count</dfn>
		<dd>
			The number of <a>grid tracks</a> crossed by a <a>grid item</a> in the applicable dimension.
	</dl>

<h3 id="algo-track-sizing">
Track Sizing Algorithm</h3>

	The remainder of this section is the <dfn>track sizing algorithm</dfn>,
	which calculates from the <a lt="min track sizing function">min</a> and <a>max track sizing functions</a>
	the used track size.
	Each track has a <dfn>base size</dfn>,
	a <<length>> which grows throughout the algorithm
	and which will eventually be the track's final size,
	and a <dfn>growth limit</dfn>,
	a <<length>> which provides a desired maximum size for the <a>base size</a>.
	There are 4 steps:

	<ol>
		<li><a href="#algo-init">Initialize Track Sizes</a>
		<li><a href="#algo-content">Resolve Intrinsic Track Sizes</a>
		<li><a href="#algo-grow-tracks">Maximize Tracks</a>
		<li><a href="#algo-flex-tracks">Stretch Flexible Tracks</a>
	</ol>

<h3 id="algo-init">
Initialize Track Sizes</h3>

	<strong>Initialize each track's base size and growth limit.</strong>
	For each track,
	if the track's <a>min track sizing function</a> is:

	<dl class=switch>
		<dt>A <a>fixed sizing function</a>
		<dd>
			Resolve to an absolute length and use that size as the track’s initial <a>base size</a>.

			Note: <a>Indefinite</a> lengths cannot occur,
			as they're treated as ''auto''.

		<dt>An <a>intrinsic sizing function</a>
		<dt>A <a>flexible sizing function</a>
		<dd>
			Use an initial <a>base size</a> of zero.
	</dl>

	For each track,
	if the track's <a>max track sizing function</a> is:

	<dl class=switch>
		<dt>A <a>fixed sizing function</a>
		<dd>
			Resolve to an absolute length and use that size as the track’s initial <a>growth limit</a>.

		<dt>An <a>intrinsic sizing function</a>
		<dd>
			Use an initial <a>growth limit</a> of infinity.

		<dt>A <a>flexible sizing function</a>
		<dd>
			Use the track’s initial <a>base size</a> as its initial <a>growth limit</a>.
	</dl>

	In all cases, if the <a>growth limit</a> is less than the <a>base size</a>,
	increase the <a>growth limit</a> to match the <a>base size</a>.

	Note: <a>Gutters</a> are treated as fixed-size tracks for the purpose of the <a>track sizing algorithm</a>.

<h3 id="algo-content">
Resolve Intrinsic Track Sizes</h3>

	This step resolves intrinsic track <a>sizing functions</a> to absolute lengths.
	First it resolves those sizes based on items that are contained wholly within a single track.
	Then it gradually adds in the space requirements of items that span multiple tracks,
	evenly distributing the extra space across those tracks
	insofar as possible.

	When the <a>grid container</a> is being sized under a <a>min-content constraint</a>,
	a track with a flexible <a>min track sizing function</a>
	is treated as if its <a>min track sizing function</a> was ''min-content''
	for the purposes of this step.
	<span class='issue'>This seems correct, but should be checked because it wasn't in the original algorithm.</span>

	Note: There is no single way to satisfy these constraints
	when items span across multiple tracks.
	This algorithm embodies a number of heuristics
	which have been seen to deliver good results on real-world use-cases,
	such as the "game" examples earlier in this specification.
	This algorithm may be updated in the future
	to take into account more advanced heuristics as they are identified.

	<ol>
		<li>
			<strong>Size tracks to fit non-spanning items:</strong>
			For each track with an intrinsic <a>track sizing function</a>,
			consider the items in it with a span of 1:
			<ul>
				<li>
					<strong>For min-content minimums:</strong>
					If the track has a ''min-content'' <a>min track sizing function</a>,
					set its <a>base size</a>
					to the maximum of the items’ <a>min-content contributions</a>.
				<li>
					<strong>For max-content minimums:</strong>
					If the track has a ''max-content'' <a>min track sizing function</a>,
					set its <a>base size</a>
					to the maximum of the items’ <a>max-content contributions</a>.
				<li>
					<strong>For auto minimums:</strong>
					If the track has an ''auto'' <a>min track sizing function</a>,
					set its <a>base size</a>
					to the maximum of the items’ outer minimum sizes,
					as specified by their respective 'min-width' or 'min-height' properties
					(whichever matches the relevant axis).
				<li>
					<strong>For min-content maximums:</strong>
					If the track has a ''min-content'' <a>max track sizing function</a>,
					set its <a>growth limit</a>
					to the maximum of the items’ <a>min-content contributions</a>.
				<li>
					<strong>For max-content and auto maximums:</strong>
					If the track has a ''max-content'' or ''auto'' <a>max track sizing function</a>,
					set its <a>growth limit</a>
					to the maximum of the items’ <a>max-content contributions</a>.
			</ul>

			In all cases, if a track’s <a>growth limit</a> is now less than its <a>base size</a>,
			increase the <a>growth limit</a> to match the <a>base size</a>.

		<li>
			<strong>Increase sizes to accommodate spanning items:</strong>
			Next, consider the items with a span of 2
			that do not span a track with a <a>flexible sizing function</a>:
			<!-- auto-min contribution <= min-content contribution <= max-content contribution -->
			<ol>
				<li>
					<strong>For intrinsic minimums:</strong>
					First increase the <a>base size</a> of tracks with
					a <a>min track sizing function</a> of ''min-content'', ''max-content'', or ''auto''
					by <a href="#extra-space">distributing extra space</a> as needed
					to accommodate the contributions dictated by these items’
					minimum size (as specified by 'min-width'/'min-height').
				<li>
					<strong>For content-based minimums:</strong>
					Next continue to increase the <a>base size</a> of tracks with
					a <a>min track sizing function</a> of ''min-content'' or ''max-content''
					by <a href="#extra-space">distributing extra space</a> as needed
					to account for these items' <a>min-content contributions</a>.
				<li>
					<strong>For max-content minimums:</strong>
					Third continue to increase the <a>base size</a> of tracks with
					a <a>min track sizing function</a> of ''max-content''
					by <a href="#extra-space">distributing extra space</a> as needed
					to account for these items' <a>max-content contributions</a>.
				<li>
					If at this point any track’s <a>growth limit</a> is now less than its <a>base size</a>,
					increase its <a>growth limit</a> to match its <a>base size</a>.
				<li>
					<strong>For intrinsic maximums:</strong>
					Next increase the <a>growth limit</a> of tracks with
					a <a>max track sizing function</a> of ''min-content'' or ''max-content'' (including ''auto'')
					by <a href="#extra-space">distributing extra space</a> as needed
					to account for these items' <a>min-content contributions</a>.
					Mark any tracks whose <a>growth limit</a> changed
					from infinite to finite in this step
					as <dfn>infinitely growable</dfn> for the next step.

					<details class=why>
						<summary>Why does the <a>infinitely growable</a> flag exist?</summary>

						<a href="http://lists.w3.org/Archives/Public/www-style/2014Mar/0500.html">Peter Salas explains</a>:

						<blockquote cite="http://lists.w3.org/Archives/Public/www-style/2014Mar/0500.html">
							Consider the following case:

							Two "auto" tracks (i.e. "minmax(min-content, max-content) minmax(min-content, max-content)").
							Item 1 is in track 1, and has min-content = max-content = 10.
							Item 2 spans tracks 1 and 2, and has min-content = 30, max-content = 100.

							After resolving min-content/max-content for the first item, we have this.

							track 1: used breadth = 10 max breadth = 10

							track 2: used breadth = 0 max breadth = infinity

							Then we resolve min-content/max-content for the second item.

							Phase 1 sets the used breadth of track 2 to 20 so that the two tracks' used breadths sum to 30.
							Phase 2 does nothing because there are no relevant tracks.
							Phase 3 sets the max breadth of track 2 to 20 so that the two tracks' max breadths sum to 30.
							In phase 4, we need to grow the sum of the max breadths by 70 to accommodate item 2.
							Two options are:

							1. Grow each track's max breadth equally,
								and end up with max breadths = [45, 55].
							2. Grow only the second track's max breadth,
								and end up with max breadths = [10, 90].

							By not considering the just-set max breadth as a constraint during space distribution
							(i.e. by treating it as infinity),
							we get the second result,
							which we considered a better result because the first track remains sized exactly to the first item.
						</blockquote>
					</details>
				<li>
					<strong>For max-content maximums:</strong>
					Lastly continue to increase the <a>growth limit</a> of tracks with
					a <a>max track sizing function</a> of ''max-content'' (including ''auto'')
					by <a href="#extra-space">distributing extra space</a> as needed
					to account for these items' <a>max-content contributions</a>.
			</ol>

			Repeat incrementally for items with greater spans until all items have been considered.

		<li>
			If any track still has an infinite <a>growth limit</a>
			(because, for example, it had no items placed in it),
			set its <a>growth limit</a> to its <a>base size</a>.
	</ol>

	<p id='extra-space'>
		To <dfn noexport>distribute extra space</dfn>
		by increasing the affected sizes of a set of tracks
		as required by a set of intrinsic size contributions,

	<ol>
		<li>
			Maintain separately for each affected <a>base size</a> or <a>growth limit</a>
			an amount of planned increase.
			(This prevents the size increases from becoming order-dependent.)

		<li>
			For each considered item,
			<ol>
				<li>
					<strong>Find the space to distribute:</strong>
					Subtract the corresponding size (<a>base size</a> or <a>growth limit</a>) of <em>every</em> spanned track
					from the item's size contribution to find the item's remaining size contribution.
					(For infinite <a>growth limits</a>, substitute the track's <a>base size</a>.)
					This is the space to distribute. Floor it at zero.
					<pre><var>extra-space</var> = max(0, <var>size-contribution</var> - ∑<var>track-sizes</var>)</pre>

				<li>
					<strong>Distribute space to base sizes up to growth limits:</strong>
					Distribute the space equally to the planned increase of each spanned track with an affected size,
					freezing tracks as their planned size reaches their <a>growth limits</a>
					(and continuing to grow the unfrozen tracks as needed).

					If a track was marked as <a>infinitely growable</a> for this phase,
					treat its <a>growth limit</a> as infinite for this calculation
					(and then unmark it).

					Note: If the affected size was a <a>growth limit</a>,
					this step has no effect.

				<li>
					<strong>Distribute space beyond growth limits:</strong>
					If space remains after all tracks are frozen,
					unfreeze and continue to distribute space to…

					<ul>
						<li>
							when handling ''min-content'' or ''auto'' <a>base sizes</a>:
							any affected track that happens to also have an intrinsic <a>max track sizing function</a>;
							if there are no such tracks, then all affected tracks.
						<li>
							when handling ''max-content'' <a>base sizes</a>:
							any affected track that happens to also have a ''max-content'' <a>max track sizing function</a>;
							if there are no such tracks, then all affected tracks.
						<li>when handling any intrinsic <a>growth limit</a>:
							all affected tracks.
					</ul>
				<li>
					<strong>Update the tracks' affected sizes</strong>
					by folding in the calculated increase
					so that the next round of space distribution will account for the increase.
					(If the affected size is infinite,
					set it to the track's <a>base size</a> plus the calculated increase.)
			</ol>
	</ol>

	Note: When this step is complete,
	all intrinsic <a>base sizes</a> and <a>growth limits</a>
	will have been resolved to absolute lengths.


<h3 id="algo-grow-tracks">
Maximize Tracks</h3>

	If the <a>free space</a> is positive, distribute it equally to all tracks,
	freezing tracks as they reach their <a>growth limits</a>
	(and continuing to grow the unfrozen tracks as needed).

	For the purpose of this step:
	if sizing the <a>grid container</a> under a <a>max-content constraint</a>,
	the <a>free space</a> is infinite;
	if sizing under a <a>min-content constraint</a>,
	the <a>free space</a> is zero.

<h3 id="algo-flex-tracks">
Stretch Flexible Tracks</h3>

	This step sizes flexible tracks
	using the largest value it can assign to an ''fr''
	without exceeding the <a>available space</a>.

	First, find the used <a>flex fraction</a>:
	<dl>
		<dt>If the <a>free space</a> is zero:
		<dd>
			The used <a>flex fraction</a> is zero.

		<dt>If the <a>free space</a> is a <a>definite</a> length:
		<dd>
			The used <a>flex fraction</a> is the result of
			<a href="#algo-find-fr-size">finding the size of an fr</a>
			using all of the <a>grid tracks</a>
			and a <a>space to fill</a> of the <a>available space</a>.

		<dt>If the <a>free space</a> is an <a>indefinite</a> length:
		<dd>
			The used <a>flex fraction</a> is the maximum of:

			<ul>
				<li>
					Each flexible track's <a>base size</a> divided by its <a>flex factor</a>.

				<li>The result of
					<a href="#algo-find-fr-size">finding the size of an fr</a>
					for each <a>grid item</a> that crosses a flexible track,
					using all the grid tracks that the item crosses
					and a <a>space to fill</a> of the item's <a>max-content contribution</a>.
			</ul>
	</dl>

	For each flexible track,
	if the product of the <var>used flex fraction</var> and the track's <a>flex factor</a>
	is greater than the track's <a>base size</a>,
	set its <a>base size</a> to that product.


<h4 id="algo-find-fr-size">
Find the Size of an ''fr''</h4>

	This algorithm finds the largest size that an ''fr'' unit can be
	without exceeding the target size.
	It must be called with a set of <a>grid tracks</a>
	and some quantity of <dfn>space to fill</dfn>.

	<ol>
		<li>
			Let <dfn>leftover space</dfn> be the <a>space to fill</a>
			minus the <a>base sizes</a> of the non-flexible <a>grid tracks</a>.

		<li>
			Let <dfn>flex factor sum</dfn> be the sum of the <a>flex factors</a> of the flexible tracks.
			If this value is less than 1,
			set it to 1 instead.

		<li>
			Let the <dfn>hypothetical fr size</dfn>
			be the <a>leftover space</a>
			divided by the <a>flex factor sum</a>.

		<li>
			If the product of the <a>hypothetical fr size</a> and a flexible track's <a>flex factor</a>
			is less than the track's base size,
			restart this algorithm
			treating all such tracks as inflexible.

		<li>
			Return the <a>hypothetical fr size</a>.
	</ol>

<!--
████████ ████████     ███     ██████   ██     ██ ████████ ██    ██ ████████ ████ ██    ██  ██████
██       ██     ██   ██ ██   ██    ██  ███   ███ ██       ███   ██    ██     ██  ███   ██ ██    ██
██       ██     ██  ██   ██  ██        ████ ████ ██       ████  ██    ██     ██  ████  ██ ██
██████   ████████  ██     ██ ██   ████ ██ ███ ██ ██████   ██ ██ ██    ██     ██  ██ ██ ██ ██   ████
██       ██   ██   █████████ ██    ██  ██     ██ ██       ██  ████    ██     ██  ██  ████ ██    ██
██       ██    ██  ██     ██ ██    ██  ██     ██ ██       ██   ███    ██     ██  ██   ███ ██    ██
██       ██     ██ ██     ██  ██████   ██     ██ ████████ ██    ██    ██    ████ ██    ██  ██████
-->

<h2 id='pagination'>
Fragmenting Grid Layout</h2>


	<p>
		<a>Grid containers</a> can break across pages
		between rows
		and inside items.
		The <a property lt="break-before">break-*</a> properties apply to grid containers
		as normal for the formatting context in which they participate.
		This section defines how they apply to grid items
		and the contents of grid items.

	<p>
		The following breaking rules refer to the fragmentation container as the “page”.
		The same rules apply to any other fragmentation containers.
		(Substitute “page” with the appropriate fragmentation container type as needed.)
		See the CSS3 Fragmentation Module [[!CSS3-BREAK]].

	<p>
		The exact layout of a fragmented grid container is not defined in this level of Grid Layout.
		However, breaks inside a grid container are subject to the following rules:

	<ul>
		<li>
			The 'break-before' and 'break-after' properties on flex items
			are propagated to their grid row.
			The 'break-before' property on the first row
			and the 'break-after' property on the last row
			are propagated to the grid container.

		<li>
			A forced break inside a grid item effectively increases the size of its contents;
			it does not trigger a forced break inside sibling items.

		<li>
			<a href="http://www.w3.org/TR/css3-break/#btw-blocks">Class A break opportunities</a> occur between rows,
			and <a href="http://www.w3.org/TR/css3-break/#end-block">Class C break opportunities</a> occur between the first/last row and the flex container's content edges.
			[[!CSS3-BREAK]]

		<li>
			When a grid container is continued after a break,
			the space available to its <a>grid items</a>
			(in the block flow direction of the fragmentation context)
			is reduced by the space consumed by grid container fragments on previous pages.
			The space consumed by a grid container fragment is
			the size of its content box on that page.
			If as a result of this adjustment the available space becomes negative,
			it is set to zero.

		<li>
			Aside from the rearrangement of items imposed by the previous point,
			UAs should attempt to minimize distortation of the grid container
			with respect to unfragmented flow.
	</ul>


<h3 id="fragmentation-alg">
Sample Fragmentation Algorithm</h3>

	<p class="issue">
		This is just a rough draft.
		This section needs to be severely cross-checked with the [[CSS3-FLEXBOX]] algorithm.
		Feedback overall is welcome; please reference the rules above instead as implementation guidance.

	<ol>
		<li>
			Layout the grid following the <a section href="#layout-algorithm"></a> by using the
			fragmentainer box width and assume unlimited height. During this step all 'grid-row'
			''auto'' and ''fr'' values must be resolved.

		<li>
			Layout the grid container using the values resolved in the previous step.

		<li>
			If a grid area dimensions change due to fragmentation (do not include items that
			span rows in this decision), increase the grid row as necessary for rows that either:
			<ul>
				<li>
					have a content min track sizing function.

				<li>
					are in a grid that does not have an explicit height and the grid row is flexible.
			</ul>

		<li>
			If the grid height is ''auto'', the height of the grid should be the sum of the final
			row sizes.

		<li>
			If a grid area overflows the grid container due to margins being collapsed during
			fragmentation, extend the grid container to contain this grid area (this step is
			necessary in order to avoid circular layout dependencies due to fragmentation).
	</ol>

	<p class="note">
		If the grid’s height is specified, steps three and four may cause the grid rows to
		overflow the grid.
	</p>

<h2 class="no-num" id="acks">
Acknowledgements</h2>

	This specification is made possible by input from
	Erik Anderson,
	Rossen Atanassov,
	Manuel Rego Casasnovas,
	Arron Eicholz,
	Sylvain Galineau,
	Markus Mielke,
	Daniel Holbert,
	John Jansen,
	Chris Jones,
	Kathy Kam,
	Veljko Miljanic,
	Mats Palmgren,
	François Remy,
	Sergio Villar Senin,
	Christian Stockwell,
	Eugene Veselov,
	and the CSS Working Group members,
	with special thanks to
	Rossen Atanassov, Alex Mogilevsky, Phil Cupp, and Peter Salas of Microsoft for creating the initial proposal.
	Thanks also to Eliot Graff for editorial input.

<h2 class="no-num" id="changes">
Changes</h2>

	The following significant changes were made since the
	<a href=http://www.w3.org/TR/2015/WD-css-grid-1-20150806/>6 August 2015 Working Draft</a>.

	* Renamed <css>row-gap</css>/<css>column-gap</css> to 'grid-row-gap'/'grid-column-gap'
		to avoid having multicol and grid share 'column-gap'.
		This also preserves <css>grid-</css> as a prefix across these properties and their shorthand.
	* Split ''auto'' into ''auto-fill'' and ''auto-fit'' keywords for ''repeat()'' notation,
		adjusted grammar to guarantee a definite number of repetitions,
		and allowed minimum sizes to increase the number of repetitions past one to fulfill the minimum.
	* Reduced <<auto-repeat>> syntax to accept only a single track size, for simplicity.
	* Added information on the speech and navigation impact of reordering with grid placement or 'order'
		vs. reordering the underlying source.

	A partial <a href="http://dev.w3.org/csswg/css-grid-1/issues-wd-20150805">Disposition of Comments</a> is available.

	See also <a href="http://www.w3.org/TR/2015/WD-css-grid-1-20150806/#changes">previous changes</a>.

<h2 class="no-num" id="translated-algo">
Appendix A: Translated Layout Algorithm</h2>

	Note: The following is a more direct translation of the
	<a href="http://www.w3.org/TR/2014/WD-css-grid-1-20140123/#track-sizing-algorithm">original Microsoft editors' track sizing algorithm</a>,
	from which the current algorithm is derived.
	It follows the original algorithm structure
	without being as programmatic in its language.
	It is preserved here to aid in review of the current specification
	(and will be removed in the next revision).

	This algorithm is run multiple times,
	once for each set of <a>grid items</a> with a particular <a>span count</a>,
	and each time it moves through four phases,
	each one addressing one combination of ''min-content'' or ''max-content''
	as a <a>min track sizing function</a> or <a>max track sizing function</a>.

	Starting from the set of all <a>grid items</a> in the grid,
	remove any with a <a>span count</a> greater than one
	which span a track with a <<flex>> <a>max track sizing function</a>.
	Of the remaining,
	sort them into groups according to their <a>span count</a>,
	and order the groups in ascending order of <a>span count</a>.
	For each group, in order,
	perform the following steps four times:

	<ol>
		<li>
			<b>Initialize variables</b>.

			Let each item's <dfn>needed space</dfn> be:

			<dl class="phase">
				<dt>In phase 1
				<dd>
					The min-content size of the item,
					minus the sum of the <a>base sizes</a> of the grid tracks it spans.
				<dt>In phase 2
				<dd>
					The max-content size of the item,
					minus the sum of the <a>base sizes</a> of the grid tracks it spans.
				<dt>In phase 3
				<dd>
					The min-content size of the item,
					minus the sum of the <a>growth limits</a> of the grid tracks it spans
					(or, if the <a>growth limit</a> is infinite, the <a>base size</a>).
				<dt>In phase 4
				<dd>
					The max-content size of the item,
					minus the sum of the <a>growth limits</a> of the grid tracks it spans
					(or, if the <a>growth limit</a> is infinite, the <a>base size</a>).
			</dl>

			Let each item's <dfn>growable tracks</dfn> be:

			<dl class="phase">
				<dt>In phase 1
				<dd>
					The grid tracks it spans with a ''min-content'' or ''max-content'' <a>min track sizing function</a>.
				<dt>In phase 2
				<dd>
					The grid tracks it spans with a ''max-content'' <a>min track sizing function</a>.
				<dt>In phase 3
				<dd>
					The grid tracks it spans with a ''min-content'' or ''max-content'' <a>max track sizing function</a>.
				<dt>In phase 4
				<dd>
					The grid tracks it spans with a ''max-content'' <a>max track sizing function</a>.
			</dl>

			Let each item's <dfn>overgrowable tracks</dfn> be:

			<dl class="phase">
				<dt>In phase 1
				<dd>
					The item's <a>growable tracks</a> that also have a ''min-content'' or ''max-content'' <a>max track sizing function</a>.
					If there are no such tracks,
					then it's all the <a>growable tracks</a>.
				<dt>In phase 2
				<dd>
					The item's <a>growable tracks</a> that also have a ''max-content'' <a>max track sizing function</a>.
					If there ar eno such tracks,
					then it's all the <a>growable tracks</a>.
				<dt>In phase 3
				<dt>In phase 4
				<dd>
					The item's <a>growable tracks</a>.
			</dl>

			Let each track's <dfn>growth delta</dfn> initially be zero.

		<li>
			<b>Calculate growth deltas.</b>

			For each grid item in the group:

			<ol>
				<li>
					<dl class="phase">
						<dt>In phase 1
						<dt>In phase 2
						<dd>
							Let each track's <dfn>hypothetical size</dfn>
							be the track's <a>base size</a>.
						<dt>In phase 3
						<dt>In phase 4
						<dd>
							Let each track's <a>hypothetical size</a>
							be the track's <a>growth limit</a> if it's finite,
							or else the track's <a>base size</a> otherwise.
					</dl>

				<li>
					Distribute the item's <a>needed space</a>
					as evenly as possible among the <a>hypothetical sizes</a> of the item's <a>growable tracks</a>,
					maxing each out as its <a>hypothetical size</a> reaches its <a>growth limit</a>.

					<dl class="phase">
						<dt>In phase 4
						<dd>
							If the track's <a>growth limit</a> was changed from infinite to a finite value
							in phase 3,
							treat its <a>growth limit</a> as infinite for the purposes of this step.
					</dl>

				<li>
					If all <a>growable tracks</a> reached their <a>growth limit</a> in the previous step
					and there is still <a>needed space</a> to distribute,
					distribute the leftover space evenly among the <a>hypothetical sizes</a> of the item's <a>overgrowable tracks</a>.

				<li>
					<dl class="phase">
						<dt>In phase 1
						<dt>In phase 2
						<dd>
							For each track,
							if the difference between its <a>hypothetical size</a> and its <a>base size</a>
							is greater than its <a>growth delta</a>,
							set its <a>growth delta</a> to that difference.
						<dt>In phase 3
						<dt>In phase 4
						<dd>
							For each track,
							if its <a>growth limit</a> is finite,
							and the difference between its <a>hypothetical size</a> and its <a>growth limit</a>
							is greater than its <a>growth delta</a>,
							set its <a>growth delta</a> to that difference.
							Otherwise, if its <a>growth limit</a> is infinite,
							and the difference between its <a>hypothetical size</a> and its <a>base size</a>
							is greater than its <a>growth delta</a>,
							set its <a>growth delta</a> to that difference.
					</dl>
			</ol>

		<li>
			<b>Adjust each track's affected size.</b>

			<dl class="phase">
				<dt>In phase 1
				<dt>In phase 2
				<dd>
					For each track,
					increase its <a>base size</a> by its <a>growth delta</a>.
				<dt>In phase 3
				<dt>In phase 4
				<dd>
					For each track,
					if its <a>growth limit</a> is finite,
					increase its <a>growth limit</a> by its <a>growth delta</a>.
					Otherwise, if its <a>growth limit</a> is infinite,
					set its <a>growth limit</a>
					to the sum of its <a>base size</a> and its <a>growth delta</a>.
			</dl>
	</ol>
